@@ -1,16 +1,14 @@
-*
       SUBROUTINE PCHEEV( JOBZ, UPLO, N, A, IA, JA, DESCA, W, Z, IZ, JZ,
      $                   DESCZ, WORK, LWORK, RWORK, LRWORK, INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
-*
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     November 15, 1997
+*     August 14, 2001 
 *
 *     .. Scalar Arguments ..
       CHARACTER          JOBZ, UPLO
-      INTEGER            IA, INFO, IZ, JA, JZ, LWORK, LRWORK, N
+      INTEGER            IA, INFO, IZ, JA, JZ, LRWORK, LWORK, N
 *     ..
 *     .. Array Arguments ..
       INTEGER            DESCA( * ), DESCZ( * )
@@ -19,14 +17,13 @@
 *     ..
 *
 *  Purpose
-*
 *  =======
 *
 *  PCHEEV computes selected eigenvalues and, optionally, eigenvectors
 *  of a real symmetric matrix A by calling the recommended sequence
 *  of ScaLAPACK routines.
 *
-*  In it's present form, PCHEEV assumes a homogeneous system and makes
+*  In its present form, PCHEEV assumes a homogeneous system and makes
 *  only spot checks of the consistency of the eigenvalues across the
 *  different processes.  Because of this, it is possible that a
 *  heterogeneous system may return incorrect results without any error
@@ -133,7 +130,8 @@
 *          eigenvalues in ascending order.
 *
 *  Z       (local output) COMPLEX array,
-*          global dimension (N, N), local dimension ( LLD_Z, LOCc(JZ+N-1) )
+*          global dimension (N, N),
+*          local dimension (LLD_Z, LOCc(JZ+N-1))
 *          If JOBZ = 'V', then on normal exit the first M columns of Z
 *          contain the orthonormal eigenvectors of the matrix
 *          corresponding to the selected eigenvalues.
@@ -184,7 +182,8 @@
 *
 *  RWORK   (local workspace/output) COMPLEX array,
 *          dimension (LRWORK)
-*          On output RWORK(1) returns the real workspace needed to
+*          On output RWORK(1) returns the
+*          REAL workspace needed to
 *          guarantee completion.  If the input parameters are incorrect,
 *          RWORK(1) may also be incorrect.
 *
@@ -242,9 +241,9 @@
       PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,
      $                   CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
      $                   RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
-      REAL               ONE, ZERO
+      REAL               ZERO, ONE
       PARAMETER          ( ZERO = 0.0E+0, ONE = 1.0E+0 )
-      COMPLEX            CONE, CZERO
+      COMPLEX            CZERO, CONE
       PARAMETER          ( CZERO = ( 0.0E+0, 0.0E+0 ),
      $                   CONE = ( 1.0E+0, 0.0E+0 ) )
       INTEGER            ITHVAL
@@ -252,16 +251,16 @@
 *     ..
 *     .. Local Scalars ..
       LOGICAL            LOWER, WANTZ
-      INTEGER            CONTEXTC, CSRC_A, I, IACOL, IAROW, ICOFFA, 
-     $                   IINFO, INDD, INDE, INDRD, INDRE, INDRWORK, 
-     $                   INDTAU, INDWORK, INDWORK2, IROFFA, IROFFZ, 
-     $                   ISCALE, IZROW, J, K, LDC, LLRWORK, LLWORK, 
-     $                   LRWMIN, LWMIN, MB_A, MB_Z, MYCOL, MYPCOLC, 
-     $                   MYPROWC, MYROW, NB, NB_A, NB_Z, NP0, NPCOL, 
-     $                   NPCOLC, NPROCS, NPROW, NPROWC, NQ0, NRC, 
-     $                   RSIZECSTEQR2, RSRC_A, RSRC_Z, SIZECSTEQR2, 
+      INTEGER            CONTEXTC, CSRC_A, I, IACOL, IAROW, ICOFFA,
+     $                   IINFO, INDD, INDE, INDRD, INDRE, INDRWORK,
+     $                   INDTAU, INDWORK, INDWORK2, IROFFA, IROFFZ,
+     $                   ISCALE, IZROW, J, K, LDC, LLRWORK, LLWORK,
+     $                   LRMIN, LRWMIN, LWMIN, MB_A, MB_Z, MYCOL,
+     $                   MYPCOLC, MYPROWC, MYROW, NB, NB_A, NB_Z, NP0,
+     $                   NPCOL, NPCOLC, NPROCS, NPROW, NPROWC, NQ0, NRC,
+     $                   RSIZECSTEQR2, RSRC_A, RSRC_Z, SIZECSTEQR2,
      $                   SIZEPCHETRD, SIZEPCUNMTR
-      REAL               ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA, 
+      REAL               ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA,
      $                   SMLNUM
 *     ..
 *     .. Local Arrays ..
@@ -270,18 +269,19 @@
 *     .. External Functions ..
       LOGICAL            LSAME
       INTEGER            INDXG2P, NUMROC, SL_GRIDRESHAPE
-      DOUBLE PRECISION   PDLAMCH, PCLANHE
-      EXTERNAL           INDXG2P, LSAME, NUMROC, PCLANHE, PSLAMCH, 
-     $                   SL_GRIDRESHAPE
+      REAL               PCLANHE, PSLAMCH
+      EXTERNAL           LSAME, INDXG2P, NUMROC, SL_GRIDRESHAPE,
+     $                   PCLANHE, PSLAMCH
 *     ..
 *     .. External Subroutines ..
-      EXTERNAL           BLACS_GRIDEXIT, BLACS_GRIDINFO, CHK1MAT, 
-     $                   CSTEQR2, DESCINIT, PCELGET, PCGEMR2D, PCHETRD, 
-     $                   PCHK1MAT, PCHK2MAT, PCLASCL, PCLASET, PCUNMTR, 
+      EXTERNAL           BLACS_GRIDEXIT, BLACS_GRIDINFO, CHK1MAT,
+     $                   CSTEQR2, DESCINIT, PCELGET, PCGEMR2D, PCHETRD,
+     $                   PCHK1MAT, PCHK2MAT, PCLASCL, PCLASET, PCUNMTR,
      $                   PXERBLA, SCOPY, SGAMN2D, SGAMX2D, SSCAL
 *     ..
 *     .. Intrinsic Functions ..
-      INTRINSIC          CMPLX, ICHAR, MAX, MIN, MOD, REAL, SQRT
+      INTRINSIC          ABS, CMPLX, ICHAR, INT, MAX, MIN, MOD, REAL,
+     $                   SQRT
 *     ..
 *     .. Executable Statements ..
 *       This is just to keep ftnchek and toolpack/1 happy
@@ -297,6 +297,18 @@
 *
       CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYROW, MYCOL )
       INFO = 0
+*
+*     Initialize pointer to some safe value
+*
+      INDTAU = 1
+      INDD = 1
+      INDE = 1
+      INDWORK = 1
+      INDWORK2 = 1
+*
+      INDRE = 1
+      INDRD = 1
+      INDRWORK = 1
 *
       WANTZ = LSAME( JOBZ, 'V' )
       IF( NPROW.EQ.-1 ) THEN
@@ -306,7 +318,7 @@
             INFO = -( 1200+CTXT_ )
          END IF
       END IF
-      IF( INFO .EQ. 0 ) THEN
+      IF( INFO.EQ.0 ) THEN
          CALL CHK1MAT( N, 3, N, 3, IA, JA, DESCA, 7, INFO )
          IF( WANTZ )
      $      CALL CHK1MAT( N, 3, N, 3, IZ, JZ, DESCZ, 12, INFO )
@@ -315,8 +327,8 @@
 *
 *           Get machine constants.
 *
-            SAFMIN = PDLAMCH( DESCA( CTXT_ ), 'Safe minimum' )
-            EPS = PDLAMCH( DESCA( CTXT_ ), 'Precision' )
+            SAFMIN = PSLAMCH( DESCA( CTXT_ ), 'Safe minimum' )
+            EPS = PSLAMCH( DESCA( CTXT_ ), 'Precision' )
             SMLNUM = SAFMIN / EPS
             BIGNUM = ONE / SMLNUM
             RMIN = SQRT( SMLNUM )
@@ -344,40 +356,32 @@
                IZROW = INDXG2P( 1, NB_A, MYROW, RSRC_Z, NPROW )
             END IF
 *
-*           Complex work space for PCHETRD
+*           COMPLEX work space for PCHETRD
 *
             CALL PCHETRD( UPLO, N, A, IA, JA, DESCA, RWORK( INDD ),
      $                    RWORK( INDE ), WORK( INDTAU ),
      $                    WORK( INDWORK ), -1, IINFO )
-            SIZEPCHETRD = WORK( 1 )
+            SIZEPCHETRD = INT( ABS( WORK( 1 ) ) )
 *
-*           Complex work space for PCUNMTR
+*           COMPLEX work space for PCUNMTR
 *
             IF( WANTZ ) THEN
                CALL PCUNMTR( 'L', UPLO, 'N', N, N, A, IA, JA, DESCA,
      $                       WORK( INDTAU ), Z, IZ, JZ, DESCZ,
      $                       WORK( INDWORK ), -1, IINFO )
-               SIZEPCUNMTR = WORK( 1 )
+               SIZEPCUNMTR = INT( ABS( WORK( 1 ) ) )
             ELSE
                SIZEPCUNMTR = 0
             END IF
 *
-*           Real work space for CSTEQR2
+*           REAL work space for CSTEQR2
 *
             IF( WANTZ ) THEN
-               RSIZECSTEQR2 = MIN( 1, 2*N - 2 )
+               RSIZECSTEQR2 = MIN( 1, 2*N-2 )
             ELSE
                RSIZECSTEQR2 = 0
             END IF
 *
-*           Complex work space for CSTEQR2
-*
-            IF( WANTZ ) THEN
-               SIZECSTEQR2 = N*LDC
-            ELSE
-               SIZECSTEQR2 = 0
-            END IF
-*
 *           Initialize the context of the single column distributed
 *           matrix required by CSTEQR2. This specific distribution
 *           allows each process to do 1/pth of the work updating matrix
@@ -392,8 +396,16 @@
      $                              MYPCOLC )
                NRC = NUMROC( N, NB_A, MYPROWC, 0, NPROCS )
                LDC = MAX( 1, NRC )
-               CALL DESCINIT( DESCQR, N, N, NB, NB, 0, 0, CONTEXTC,
-     $                        LDC, INFO )
+               CALL DESCINIT( DESCQR, N, N, NB, NB, 0, 0, CONTEXTC, LDC,
+     $                        INFO )
+            END IF
+*
+*           COMPLEX work space for CSTEQR2
+*
+            IF( WANTZ ) THEN
+               SIZECSTEQR2 = N*LDC
+            ELSE
+               SIZECSTEQR2 = 0
             END IF
 *
 *           Set up pointers into the WORK array
@@ -471,8 +483,8 @@
          END IF
          IDUM2( 3 ) = 3
          IF( WANTZ ) THEN
-            CALL PCHK2MAT( N, 3, N, 3, IA, JA, DESCA, 7, N, 3, N, 3,
-     $                     IZ, JZ, DESCZ, 12, 3, IDUM1, IDUM2, INFO )
+            CALL PCHK2MAT( N, 3, N, 3, IA, JA, DESCA, 7, N, 3, N, 3, IZ,
+     $                     JZ, DESCZ, 12, 3, IDUM1, IDUM2, INFO )
          ELSE
             CALL PCHK1MAT( N, 3, N, 3, IA, JA, DESCA, 7, 3, IDUM1,
      $                     IDUM2, INFO )
@@ -496,8 +508,8 @@
 *
       ISCALE = 0
 *
-      ANRM = PCLANHE( 'M', UPLO, N, A, IA, JA, DESCA, 
-     $                RWORK( INDRWORK ) )
+      ANRM = PCLANHE( 'M', UPLO, N, A, IA, JA, DESCA,
+     $       RWORK( INDRWORK ) )
 *
 *
       IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
@@ -509,8 +521,7 @@
       END IF
 *
       IF( ISCALE.EQ.1 ) THEN
-         CALL PCLASCL( UPLO, ONE, SIGMA, N, N, A, IA, JA, DESCA,
-     $                 IINFO )
+         CALL PCLASCL( UPLO, ONE, SIGMA, N, N, A, IA, JA, DESCA, IINFO )
       END IF
 *
 *     Reduce symmetric matrix to tridiagonal form.
@@ -595,22 +606,26 @@
          K = ITHVAL
       END IF
 *
+      LRMIN = INT( RWORK( 1 ) )
+      INDTAU = 0
+      INDE = INDTAU + J
       DO 40 I = 1, J
-         WORK( I+INDTAU ) = W( ( I-1 )*K+1 )
-         WORK( I+INDE ) = W( ( I-1 )*K+1 )
+         RWORK( I+INDTAU ) = W( ( I-1 )*K+1 )
+         RWORK( I+INDE ) = W( ( I-1 )*K+1 )
    40 CONTINUE
 *
-      CALL SGAMN2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDTAU ),
+      CALL SGAMN2D( DESCA( CTXT_ ), 'All', ' ', J, 1, RWORK( 1+INDTAU ),
+     $              J, 1, 1, -1, -1, 0 )
+      CALL SGAMX2D( DESCA( CTXT_ ), 'All', ' ', J, 1, RWORK( 1+INDE ),
      $              J, 1, 1, -1, -1, 0 )
-      CALL SGAMX2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDE ), J,
-     $              1, 1, -1, -1, 0 )
 *
       DO 50 I = 1, J
-         IF( INFO.EQ.0 .AND. ( WORK( I+INDTAU )-WORK( I+INDE ).NE.
+         IF( INFO.EQ.0 .AND. ( RWORK( I+INDTAU )-RWORK( I+INDE ).NE.
      $       ZERO ) ) THEN
             INFO = N + 1
          END IF
    50 CONTINUE
+      RWORK( 1 ) = LRMIN
 *
       RETURN
 *
