@@ -1,13 +1,11 @@
-*
-*
       SUBROUTINE PSGESVD( JOBU, JOBVT, M, N, A, IA, JA, DESCA, S, U, IU,
      $                    JU, DESCU, VT, IVT, JVT, DESCVT, WORK, LWORK,
      $                    INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     November 15, 1997
+*     March 22, 2000
 *
 *     .. Scalar Arguments ..
       CHARACTER          JOBU, JOBVT
@@ -307,7 +305,7 @@
      $                   SSCAL
 *     ..
 *     .. Intrinsic Functions ..
-      INTRINSIC          FLOAT, MAX, MIN, SQRT
+      INTRINSIC          MAX, MIN, REAL, SQRT
 *     ..
 *     .. Executable Statements ..
 *     This is just to keep ftnchek happy
@@ -430,7 +428,7 @@
 *           Finally, calculate required workspace.
 *
             LWMIN = 2 + 6*SIZEB + MAX( WATOBD, WBDTOSVD )
-            WORK( 1 ) = FLOAT( LWMIN )
+            WORK( 1 ) = REAL( LWMIN )
 *
             IF( WANTU.NE.1 .AND. .NOT.( LSAME( JOBU, 'N' ) ) ) THEN
                INFO = -1
