@@ -1,15 +1,13 @@
-*
-*
       SUBROUTINE PSSYGVX( IBTYPE, JOBZ, RANGE, UPLO, N, A, IA, JA,
      $                    DESCA, B, IB, JB, DESCB, VL, VU, IL, IU,
      $                    ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ,
      $                    WORK, LWORK, IWORK, LIWORK, IFAIL, ICLUSTR,
      $                    GAP, INFO )
 *
-*  -- ScaLAPACK routine (version 1.5) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     April 15, 1997
+*     October 15, 1999
 *
 *     .. Scalar Arguments ..
       CHARACTER          JOBZ, RANGE, UPLO
@@ -41,6 +39,7 @@
 *  Notes
 *  =====
 *
+*
 *  Each global data object is described by an associated description
 *  vector.  This vector stores the information required to establish
 *  the mapping between an object element and its corresponding process
@@ -92,6 +91,7 @@
 *          LOCr( M ) <= ceil( ceil(M/MB_A)/NPROW )*MB_A
 *          LOCc( N ) <= ceil( ceil(N/NB_A)/NPCOL )*NB_A
 *
+*
 *  Arguments
 *  =========
 *
@@ -136,12 +136,12 @@
 *          the diagonal, is destroyed.
 *
 *  IA      (global input) INTEGER
-*          A's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array A indicating the first
+*          row of sub( A ).
 *
 *  JA      (global input) INTEGER
-*          A's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array A indicating the
+*          first column of sub( A ).
 *
 *  DESCA   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix A.
@@ -163,12 +163,12 @@
 *          sub( B ) = L*L**T.
 *
 *  IB      (global input) INTEGER
-*          B's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array B indicating the first
+*          row of sub( B ).
 *
 *  JB      (global input) INTEGER
-*          B's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array B indicating the
+*          first column of sub( B ).
 *
 *  DESCB   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix B.
@@ -254,7 +254,8 @@
 *          ORFAC should be identical on all processes.
 *
 *  Z       (local output) REAL array,
-*          global dimension (N, N), local dimension ( LLD_Z, LOCc(JZ+N-1) )
+*          global dimension (N, N),
+*          local dimension ( LLD_Z, LOCc(JZ+N-1) )
 *          If JOBZ = 'V', then on normal exit the first M columns of Z
 *          contain the orthonormal eigenvectors of the matrix
 *          corresponding to the selected eigenvalues.  If an eigenvector
@@ -264,12 +265,12 @@
 *          If JOBZ = 'N', then Z is not referenced.
 *
 *  IZ      (global input) INTEGER
-*          Z's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array Z indicating the first
+*          row of sub( Z ).
 *
 *  JZ      (global input) INTEGER
-*          Z's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array Z indicating the
+*          first column of sub( Z ).
 *
 *  DESCZ   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix Z.
@@ -277,43 +278,22 @@
 *
 *  WORK    (local workspace/output) REAL array,
 *             dimension (LWORK)
-*          Version 1.0:  on output, WORK(1) returns the workspace
-*          needed to guarantee completion, but not orthogonality
-*          of the eigenvectors.
-*          If the input parameters are incorrect, WORK(1) may also be
-*          incorrect.
+*          if JOBZ='N' WORK(1) = optimal amount of workspace
+*             required to compute eigenvalues efficiently
+*          if JOBZ='V' WORK(1) = optimal amount of workspace
+*             required to compute eigenvalues and eigenvectors
+*             efficiently with no guarantee on orthogonality.
+*             If RANGE='V', it is assumed that all eigenvectors
+*             may be required.
 *
-*          Later we will modify this so if enough workspace is given
-*          to complete the request, WORK(1) will return the amount
-*          of workspace needed to guarantee orthogonality.  This is
-*          described in great detail below:
-*          if INFO>=0, then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' WORK(1) = minimal workspace required to
-*               guarantee orthogonal eigenvectors on the given input
-*               matrix with the given ORFAC.  (In version 1.0 WORK(1) =
-*               minimal workspace required to compute eigenvalues.)
-*          if INFO <0 then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' then
-*                if RANGE='A' or RANGE='I' then
-*                   WORK(1) = minimal workspace required to compute
-*                   all eigenvectors (no guarantee on orthogonality)
-*                if RANGE='V' then
-*                   WORK(1) = minimal workspace required to compute
-*                   N_Z = DESCZ(N_) eigenvectors (no guarantee on
-*                   orthogonality.)  (In version 1.0 WORK(1) =
-*                   minimal workspace required to compute eigenvalues.)
-*
-*  LWORK  (local input) INTEGER
-*          Size of WORK
+*  LWORK   (local input) INTEGER
 *          See below for definitions of variables used to define LWORK.
 *          If no eigenvectors are requested (JOBZ = 'N') then
 *             LWORK >= 5 * N + MAX( 5 * NN, NB * ( NP0 + 1 ) )
 *          If eigenvectors are requested (JOBZ = 'V' ) then
 *             the amount of workspace required to guarantee that all
 *             eigenvectors are computed is:
-*             LWORK >= 5*N + MAX( 5*NN, NP0 * MQ0 + 2 * NB * NB ) +
+*             LWORK >= 5 * N + MAX( 5*NN, NP0 * MQ0 + 2 * NB * NB ) +
 *               ICEIL( NEIG, NPROW*NPCOL)*NN
 *
 *             The computed eigenvectors may not be orthogonal if the
@@ -353,6 +333,40 @@
 *             eigenvalues and as many eigenvectors as it can.
 *
 *          Relationship between workspace, orthogonality & performance:
+*             Greater performance can be achieved if adequate workspace
+*             is provided.  On the other hand, in some situations,
+*             performance can decrease as the workspace provided
+*             increases above the workspace amount shown below:
+*
+*             For optimal performance, greater workspace may be
+*             needed, i.e.
+*                LWORK >=  MAX( LWORK, 5 * N + NSYTRD_LWOPT,
+*                  NSYGST_LWOPT )
+*                Where:
+*                  LWORK, as defined previously, depends upon the number
+*                     of eigenvectors requested, and
+*                  NSYTRD_LWOPT = N + 2*( ANB+1 )*( 4*NPS+2 ) +
+*                    ( NPS + 3 ) *  NPS
+*                  NSYGST_LWOPT =  2*NP0*NB + NQ0*NB + NB*NB
+*
+*                  ANB = PJLAENV( DESCA( CTXT_), 3, 'PSSYTTRD', 'L',
+*                     0, 0, 0, 0)
+*                  SQNPC = INT( SQRT( DBLE( NPROW * NPCOL ) ) )
+*                  NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+*                  NB = DESCA( MB_ )
+*                  NP0 = NUMROC( N, NB, 0, 0, NPROW )
+*                  NQ0 = NUMROC( N, NB, 0, 0, NPCOL )
+*
+*                  NUMROC is a ScaLAPACK tool functions;
+*                  PJLAENV is a ScaLAPACK envionmental inquiry function
+*                  MYROW, MYCOL, NPROW and NPCOL can be determined by
+*                    calling the subroutine BLACS_GRIDINFO.
+*
+*                For large N, no extra workspace is needed, however the
+*                biggest boost in performance comes for small N, so it
+*                is wise to provide the extra workspace (typically less
+*                than a Megabyte per process).
+*
 *             If CLUSTERSIZE >= N/SQRT(NPROW*NPCOL), then providing
 *             enough space to compute all the eigenvectors
 *             orthogonally will cause serious degradation in
@@ -368,22 +382,23 @@
 *             execution.
 *
 *          If LWORK = -1, then LWORK is global input and a workspace
-*          query is assumed; the routine only calculates the minimum
-*          and optimal size for all work arrays. Each of these
-*          values is returned in the first entry of the corresponding
-*          work array, and no error message is issued by PXERBLA.
+*          query is assumed; the routine only calculates the size
+*          required for optimal performance on all work arrays.
+*          Each of these values is returned in the first entry of the
+*          corresponding work array, and no error message is issued by
+*          PXERBLA.
+*
 *
 *  IWORK   (local workspace) INTEGER array
 *          On return, IWORK(1) contains the amount of integer workspace
 *          required.
-*          If the input parameters are incorrect, IWORK(1) may also be
-*          incorrect.
 *
 *  LIWORK  (local input) INTEGER
 *          size of IWORK
 *          LIWORK >= 6 * NNP
 *          Where:
 *            NNP = MAX( N, NPROW*NPCOL + 1, 4 )
+*
 *          If LIWORK = -1, then LIWORK is global input and a workspace
 *          query is assumed; the routine only calculates the minimum
 *          and optimal size for all work arrays. Each of these
@@ -486,9 +501,11 @@
 *     .. Local Scalars ..
       LOGICAL            ALLEIG, INDEIG, LQUERY, UPPER, VALEIG, WANTZ
       CHARACTER          TRANS
-      INTEGER            IACOL, IAROW, IBCOL, IBROW, ICOFFA, ICOFFB,
-     $                   ICTXT, IROFFA, IROFFB, LIWMIN, LWMIN, MQ0,
-     $                   MYCOL, MYROW, NB, NEIG, NN, NP0, NPCOL, NPROW
+      INTEGER            ANB, IACOL, IAROW, IBCOL, IBROW, ICOFFA,
+     $                   ICOFFB, ICTXT, IROFFA, IROFFB, LIWMIN, LWMIN,
+     $                   LWOPT, MQ0, MYCOL, MYROW, NB, NEIG, NN, NP0,
+     $                   NPCOL, NPROW, NPS, NQ0, NSYGST_LWOPT,
+     $                   NSYTRD_LWOPT, SQNPC
       REAL               EPS, SCALE
 *     ..
 *     .. Local Arrays ..
@@ -496,17 +513,18 @@
 *     ..
 *     .. External Functions ..
       LOGICAL            LSAME
-      INTEGER            ICEIL, INDXG2P, NUMROC
+      INTEGER            ICEIL, INDXG2P, NUMROC, PJLAENV
       REAL               PSLAMCH
-      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PSLAMCH
+      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PJLAENV, PSLAMCH
 *     ..
 *     .. External Subroutines ..
       EXTERNAL           BLACS_GRIDINFO, CHK1MAT, PCHK1MAT, PCHK2MAT,
-     $                   PSPOTRF, PSSYEVX, PSSYGST, PSTRMM, PSTRSM,
+     $                   PSPOTRF, PSSYEVX, PSSYNGST, PSTRMM, PSTRSM,
      $                   PXERBLA, SGEBR2D, SGEBS2D, SSCAL
 *     ..
 *     .. Intrinsic Functions ..
-      INTRINSIC          ABS, ICHAR, MAX, MIN, MOD, REAL
+      INTRINSIC          ABS, DBLE, ICHAR, INT, MAX, MIN, MOD, REAL,
+     $                   SQRT
 *     ..
 *     .. Executable Statements ..
 *       This is just to keep ftnchek and toolpack/1 happy
@@ -551,8 +569,7 @@
                   WORK( 2 ) = ZERO
                   WORK( 3 ) = ZERO
                END IF
-               CALL SGEBS2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, WORK,
-     $                       3 )
+               CALL SGEBS2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, WORK, 3 )
             ELSE
                CALL SGEBR2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, WORK, 3,
      $                       0, 0 )
@@ -585,6 +602,12 @@
             IF( ( .NOT.WANTZ ) .OR. ( VALEIG .AND. ( .NOT.LQUERY ) ) )
      $           THEN
                LWMIN = 5*N + MAX( 5*NN, NB*( NP0+1 ) )
+               IF( WANTZ ) THEN
+                  MQ0 = NUMROC( MAX( N, NB, 2 ), NB, 0, 0, NPCOL )
+                  LWOPT = 5*N + MAX( 5*NN, NP0*MQ0+2*NB*NB )
+               ELSE
+                  LWOPT = LWMIN
+               END IF
                NEIG = 0
             ELSE
                IF( ALLEIG .OR. VALEIG ) THEN
@@ -595,9 +618,23 @@
                MQ0 = NUMROC( MAX( NEIG, NB, 2 ), NB, 0, 0, NPCOL )
                LWMIN = 5*N + MAX( 5*NN, NP0*MQ0+2*NB*NB ) +
      $                 ICEIL( NEIG, NPROW*NPCOL )*NN
+               LWOPT = LWMIN
 *
             END IF
 *
+*           Conpute how much workspace is needed to use the
+*           new TRD and GST algorithms
+*
+            ANB = PJLAENV( ICTXT, 3, 'PSSYTTRD', 'L', 0, 0, 0, 0 )
+            SQNPC = INT( SQRT( DBLE( NPROW*NPCOL ) ) )
+            NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+            NSYTRD_LWOPT = 2*( ANB+1 )*( 4*NPS+2 ) + ( NPS+4 )*NPS
+            NB = DESCA( MB_ )
+            NP0 = NUMROC( N, NB, 0, 0, NPROW )
+            NQ0 = NUMROC( N, NB, 0, 0, NPCOL )
+            NSYGST_LWOPT = 2*NP0*NB + NQ0*NB + NB*NB
+            LWOPT = MAX( LWOPT, N+NSYTRD_LWOPT, NSYGST_LWOPT )
+*
 *       Version 1.0 Limitations
 *
             IF( IBTYPE.LT.1 .OR. IBTYPE.GT.3 ) THEN
@@ -651,23 +688,23 @@
             ELSE IF( VALEIG .AND. N.GT.0 .AND. VU.LE.VL ) THEN
                INFO = -15
             ELSE IF( INDEIG .AND. ( IL.LT.1 .OR. IL.GT.MAX( 1, N ) ) )
-     $               THEN
+     $                THEN
                INFO = -16
             ELSE IF( INDEIG .AND. ( IU.LT.MIN( N, IL ) .OR. IU.GT.N ) )
-     $               THEN
+     $                THEN
                INFO = -17
             ELSE IF( VALEIG .AND. ( ABS( WORK( 2 )-VL ).GT.FIVE*EPS*
-     $              ABS( VL ) ) ) THEN
+     $               ABS( VL ) ) ) THEN
                INFO = -14
             ELSE IF( VALEIG .AND. ( ABS( WORK( 3 )-VU ).GT.FIVE*EPS*
-     $              ABS( VU ) ) ) THEN
+     $               ABS( VU ) ) ) THEN
                INFO = -15
             ELSE IF( ABS( WORK( 1 )-ABSTOL ).GT.FIVE*EPS*ABS( ABSTOL ) )
-     $               THEN
+     $                THEN
                INFO = -18
-            ELSE IF( LWORK.LT.LWMIN .AND. LWORK.NE.-1 ) THEN
+            ELSE IF( LWORK.LT.LWMIN .AND. .NOT.LQUERY ) THEN
                INFO = -28
-            ELSE IF( LIWORK.LT.LIWMIN .AND. LIWORK.NE.-1 ) THEN
+            ELSE IF( LIWORK.LT.LIWMIN .AND. .NOT.LQUERY ) THEN
                INFO = -30
             END IF
          END IF
@@ -705,8 +742,8 @@
      $                  INFO )
       END IF
 *
-      WORK( 1 ) = REAL( LWMIN )
       IWORK( 1 ) = LIWMIN
+      WORK( 1 ) = REAL( LWOPT )
 *
       IF( INFO.NE.0 ) THEN
          CALL PXERBLA( ICTXT, 'PSSYGVX ', -INFO )
@@ -719,6 +756,8 @@
 *
       CALL PSPOTRF( UPLO, N, B, IB, JB, DESCB, INFO )
       IF( INFO.NE.0 ) THEN
+         IWORK( 1 ) = LIWMIN
+         WORK( 1 ) = REAL( LWOPT )
          IFAIL( 1 ) = INFO
          INFO = IERRNPD
          RETURN
@@ -726,12 +765,11 @@
 *
 *     Transform problem to standard eigenvalue problem and solve.
 *
-      CALL PSSYGST( IBTYPE, UPLO, N, A, IA, JA, DESCA, B, IB, JB,
-     $              DESCB, SCALE, INFO )
+      CALL PSSYNGST( IBTYPE, UPLO, N, A, IA, JA, DESCA, B, IB, JB,
+     $               DESCB, SCALE, WORK, LWORK, INFO )
       CALL PSSYEVX( JOBZ, RANGE, UPLO, N, A, IA, JA, DESCA, VL, VU, IL,
-     $              IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ,
-     $              WORK, LWORK, IWORK, LIWORK, IFAIL, ICLUSTR, GAP,
-     $              INFO )
+     $              IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ, WORK,
+     $              LWORK, IWORK, LIWORK, IFAIL, ICLUSTR, GAP, INFO )
 *
       IF( WANTZ ) THEN
 *
@@ -773,6 +811,8 @@
          CALL SSCAL( N, SCALE, W, 1 )
       END IF
 *
+      IWORK( 1 ) = LIWMIN
+      WORK( 1 ) = REAL( LWOPT )
       RETURN
 *
 *     End of PSSYGVX
