@@ -1,14 +1,12 @@
-*
-*
       SUBROUTINE PZHEEVX( JOBZ, RANGE, UPLO, N, A, IA, JA, DESCA, VL,
      $                    VU, IL, IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ,
      $                    JZ, DESCZ, WORK, LWORK, RWORK, LRWORK, IWORK,
      $                    LIWORK, IFAIL, ICLUSTR, GAP, INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     November 15, 1997
+*     May 25, 2001
 *
 *     .. Scalar Arguments ..
       CHARACTER          JOBZ, RANGE, UPLO
@@ -24,7 +22,6 @@
 *     ..
 *
 *  Purpose
-*
 *  =======
 *
 *  PZHEEVX computes selected eigenvalues and, optionally, eigenvectors
@@ -118,7 +115,8 @@
 *          The number of rows and columns of the matrix A.  N >= 0.
 *
 *  A       (local input/workspace) block cyclic COMPLEX*16 array,
-*          global dimension (N, N), local dimension ( LLD_A, LOCc(JA+N-1) )
+*          global dimension (N, N),
+*          local dimension ( LLD_A, LOCc(JA+N-1) )
 *
 *          On entry, the Hermitian matrix A.  If UPLO = 'U', only the
 *          upper triangular part of A is used to define the elements of
@@ -223,7 +221,8 @@
 *          ORFAC should be identical on all processes.
 *
 *  Z       (local output) COMPLEX*16 array,
-*          global dimension (N, N), local dimension ( LLD_Z, LOCc(JZ+N-1) )
+*          global dimension (N, N),
+*          local dimension ( LLD_Z, LOCc(JZ+N-1) )
 *          If JOBZ = 'V', then on normal exit the first M columns of Z
 *          contain the orthonormal eigenvectors of the matrix
 *          corresponding to the selected eigenvalues.  If an eigenvector
@@ -246,33 +245,8 @@
 *
 *  WORK    (local workspace/output) COMPLEX*16 array,
 *          dimension (LWORK)
-*          Version 1.0:  on output, WORK(1) returns the workspace
-*          needed to guarantee completion, but not orthogonality
-*          of the eigenvectors.
-*          If the input parameters are incorrect, WORK(1) may also be
-*          incorrect.
-*
-*          Later we will modify this so if enough workspace is given
-*          to complete the request, WORK(1) will return the amount
-*          of workspace needed to guarantee orthogonality.  This is
-*          described in great detail below:
-*          if INFO>=0, then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' WORK(1) = minimal workspace required to
-*               guarantee orthogonal eigenvectors on the given input
-*               matrix with the given ORFAC.  (In version 1.0 WORK(1) =
-*               minimal workspace required to compute eigenvalues.)
-*          if INFO <0 then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' then
-*                if RANGE='A' or RANGE='I' then
-*                   WORK(1) = minimal workspace required to compute
-*                   all eigenvectors (no guarantee on orthogonality)
-*                if RANGE='V' then
-*                   WORK(1) = minimal workspace required to compute
-*                   N_Z = DESCZ(N_) eigenvectors (no guarantee on
-*                   orthogonality.)  (In version 1.0 WORK(1) =
-*                   minimal workspace required to compute eigenvalues.)
+*          WORK(1) returns workspace adequate workspace to allow
+*          optimal performance.
 *
 *  LWORK   (local input) INTEGER
 *          Size of WORK array.  If only eigenvalues are requested:
@@ -281,14 +255,39 @@
 *            LWORK >= N + ( NP0 + MQ0 + NB ) * NB
 *          with NQ0 = NUMROC( NN, NB, 0, 0, NPCOL ).
 *
+*          For optimal performance, greater workspace is needed, i.e.
+*            LWORK >= MAX( LWORK, NHETRD_LWORK )
+*          Where LWORK is as defined above, and
+*          NHETRD_LWORK = N + 2*( ANB+1 )*( 4*NPS+2 ) +
+*            ( NPS + 1 ) * NPS
+*
+*          ICTXT = DESCA( CTXT_ )
+*          ANB = PJLAENV( ICTXT, 3, 'PZHETTRD', 'L', 0, 0, 0, 0 )
+*          SQNPC = SQRT( DBLE( NPROW * NPCOL ) )
+*          NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+*
+*          NUMROC is a ScaLAPACK tool functions;
+*          PJLAENV is a ScaLAPACK envionmental inquiry function
+*          MYROW, MYCOL, NPROW and NPCOL can be determined by calling
+*          the subroutine BLACS_GRIDINFO.
+*
 *          If LWORK = -1, then LWORK is global input and a workspace
-*          query is assumed; the routine only calculates the minimum
-*          and optimal size for all work arrays. Each of these
+*          query is assumed; the routine only calculates the
+*          optimal size for all work arrays. Each of these
 *          values is returned in the first entry of the corresponding
 *          work array, and no error message is issued by PXERBLA.
 *
 *  RWORK   (local workspace/output) DOUBLE PRECISION array,
 *             dimension (LRWORK)
+*          On return, WROK(1) contains the optimal amount of
+*          workspace required for efficient execution.
+*          if JOBZ='N' RWORK(1) = optimal amount of workspace
+*             required to compute eigenvalues efficiently
+*          if JOBZ='V' RWORK(1) = optimal amount of workspace
+*             required to compute eigenvalues and eigenvectors
+*             efficiently with no guarantee on orthogonality.
+*             If RANGE='V', it is assumed that all eigenvectors
+*             may be required.
 *
 *  LRWORK   (local input) INTEGER
 *          Size of RWORK
@@ -354,16 +353,15 @@
 *             execution.
 *
 *          If LRWORK = -1, then LRWORK is global input and a workspace
-*          query is assumed; the routine only calculates the minimum
-*          and optimal size for all work arrays. Each of these
-*          values is returned in the first entry of the corresponding
-*          work array, and no error message is issued by PXERBLA.
-
+*          query is assumed; the routine only calculates the size
+*          required for optimal performance for all work arrays. Each of
+*          these values is returned in the first entry of the
+*          corresponding work arrays, and no error message is issued by
+*          PXERBLA.
+*
 *  IWORK   (local workspace) INTEGER array
 *          On return, IWORK(1) contains the amount of integer workspace
 *          required.
-*          If the input parameters are incorrect, IWORK(1) may also be
-*          incorrect.
 *
 *  LIWORK  (local input) INTEGER
 *          size of IWORK
@@ -474,27 +472,28 @@
       PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,
      $                   CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
      $                   RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
-      DOUBLE PRECISION   FIVE, ONE, TEN, ZERO
-      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0,
-     $			 TEN = 10.0D+0,FIVE = 5.0D+0 )
+      DOUBLE PRECISION   ZERO, ONE, TEN, FIVE
+      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0, TEN = 10.0D+0,
+     $                   FIVE = 5.0D+0 )
       INTEGER            IERREIN, IERRCLS, IERRSPC, IERREBZ
       PARAMETER          ( IERREIN = 1, IERRCLS = 2, IERRSPC = 4,
      $                   IERREBZ = 8 )
 *     ..
 *     .. Local Scalars ..
-      LOGICAL            ALLEIG, INDEIG, LOWER, QUICKRETURN, VALEIG,
-     $                   WANTZ, LQUERY
+      LOGICAL            ALLEIG, INDEIG, LOWER, LQUERY, QUICKRETURN,
+     $                   VALEIG, WANTZ
       CHARACTER          ORDER
-      INTEGER            CSRC_A, I, IACOL, IAROW, ICOFFA, IINFO, INDD, 
-     $                   INDD2, INDE, INDE2, INDIBL, INDISP, INDRWORK, 
-     $                   INDTAU, INDWORK, IROFFA, IROFFZ, ISCALE, 
-     $                   ISIZESTEBZ, ISIZESTEIN, IZROW, LALLWORK, 
-     $                   LIWMIN, LLRWORK, LLWORK, LRWMIN, LWMIN, 
-     $                   MAXEIGS, MB_A, MB_Z, MQ0, MYCOL, MYROW, NB, 
-     $                   NB_A, NB_Z, NEIG, NN, NNP, NP0, NPCOL, NPROCS, 
-     $                   NPROW, NQ0, NSPLIT, NZZ, OFFSET, RSRC_A, 
-     $                   RSRC_Z, SIZEHEEVX, SIZEORMTR, SIZESTEIN
-      DOUBLE PRECISION   ABSTLL, ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, 
+      INTEGER            ANB, CSRC_A, I, IAROW, ICOFFA, ICTXT, IINFO,
+     $                   INDD, INDD2, INDE, INDE2, INDIBL, INDISP,
+     $                   INDRWORK, INDTAU, INDWORK, IROFFA, IROFFZ,
+     $                   ISCALE, ISIZESTEBZ, ISIZESTEIN, IZROW,
+     $                   LALLWORK, LIWMIN, LLRWORK, LLWORK, LRWMIN,
+     $                   LRWOPT, LWMIN, LWOPT, MAXEIGS, MB_A, MQ0,
+     $                   MYCOL, MYROW, NB, NB_A, NEIG, NHETRD_LWOPT, NN,
+     $                   NNP, NP0, NPCOL, NPROCS, NPROW, NPS, NQ0,
+     $                   NSPLIT, NZZ, OFFSET, RSRC_A, RSRC_Z, SIZEHEEVX,
+     $                   SIZESTEIN, SQNPC
+      DOUBLE PRECISION   ABSTLL, ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN,
      $                   SIGMA, SMLNUM, VLL, VUU
 *     ..
 *     .. Local Arrays ..
@@ -502,18 +501,20 @@
 *     ..
 *     .. External Functions ..
       LOGICAL            LSAME
-      INTEGER            ICEIL, INDXG2P, NUMROC
+      INTEGER            ICEIL, INDXG2P, NUMROC, PJLAENV
       DOUBLE PRECISION   PDLAMCH, PZLANHE
-      EXTERNAL           ICEIL, INDXG2P, LSAME, NUMROC, PDLAMCH, PZLANHE
+      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PJLAENV,
+     $                   PDLAMCH, PZLANHE
 *     ..
 *     .. External Subroutines ..
-      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DGEBR2D, DGEBS2D, 
-     $                   DLASRT, DSCAL, IGAMN2D, PCHK1MAT, PCHK2MAT, 
-     $                   PDLARED1D, PDSTEBZ, PXERBLA, PZELGET, PZHETRD, 
+      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DGEBR2D, DGEBS2D,
+     $                   DLASRT, DSCAL, IGAMN2D, PCHK1MAT, PCHK2MAT,
+     $                   PDLARED1D, PDSTEBZ, PXERBLA, PZELGET, PZHENTRD,
      $                   PZLASCL, PZSTEIN, PZUNMTR
 *     ..
 *     .. Intrinsic Functions ..
-      INTRINSIC          ABS, DBLE, DCMPLX, ICHAR, MAX, MIN, MOD, SQRT
+      INTRINSIC          ABS, DBLE, DCMPLX, ICHAR, INT, MAX, MIN, MOD,
+     $                   SQRT
 *     ..
 *     .. Executable Statements ..
 *       This is just to keep ftnchek and toolpack/1 happy
@@ -524,28 +525,29 @@
 *
 *     Test the input arguments.
 *
-      CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYROW, MYCOL )
+      ICTXT = DESCA( CTXT_ )
+      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
       INFO = 0
 *
       WANTZ = LSAME( JOBZ, 'V' )
       IF( NPROW.EQ.-1 ) THEN
-         INFO = -( 700+CTXT_ )
+         INFO = -( 800+CTXT_ )
       ELSE IF( WANTZ ) THEN
-         IF( DESCA( CTXT_ ).NE.DESCZ( CTXT_ ) ) THEN
-            INFO = -( 1200+CTXT_ )
+         IF( ICTXT.NE.DESCZ( CTXT_ ) ) THEN
+            INFO = -( 2100+CTXT_ )
          END IF
       END IF
-      IF( INFO .EQ. 0 ) THEN
-         CALL CHK1MAT( N, 3, N, 3, IA, JA, DESCA, 7, INFO )
+      IF( INFO.EQ.0 ) THEN
+         CALL CHK1MAT( N, 4, N, 4, IA, JA, DESCA, 8, INFO )
          IF( WANTZ )
-     $      CALL CHK1MAT( N, 3, N, 3, IZ, JZ, DESCZ, 12, INFO )
+     $      CALL CHK1MAT( N, 4, N, 4, IZ, JZ, DESCZ, 21, INFO )
 *
          IF( INFO.EQ.0 ) THEN
 *
 *     Get machine constants.
 *
-            SAFMIN = PDLAMCH( DESCA( CTXT_ ), 'Safe minimum' )
-            EPS = PDLAMCH( DESCA( CTXT_ ), 'Precision' )
+            SAFMIN = PDLAMCH( ICTXT, 'Safe minimum' )
+            EPS = PDLAMCH( ICTXT, 'Precision' )
             SMLNUM = SAFMIN / EPS
             BIGNUM = ONE / SMLNUM
             RMIN = SQRT( SMLNUM )
@@ -583,7 +585,7 @@
 *
             LQUERY = .FALSE.
             IF( LWORK.EQ.-1 .OR. LIWORK.EQ.-1 .OR. LRWORK.EQ.-1 )
-     $          LQUERY = .TRUE.
+     $         LQUERY = .TRUE.
 *
             NNP = MAX( N, NPROCS+1, 4 )
             LIWMIN = 6*NNP
@@ -599,20 +601,26 @@
             IROFFA = MOD( IA-1, MB_A )
             ICOFFA = MOD( JA-1, NB_A )
             IAROW = INDXG2P( 1, NB_A, MYROW, RSRC_A, NPROW )
-            IACOL = INDXG2P( 1, MB_A, MYCOL, CSRC_A, NPCOL )
-            NP0 = NUMROC( N+IROFFA, NB, MYROW, IAROW, NPROW )
-            MQ0 = NUMROC( N+ICOFFA, NB, MYCOL, IACOL, NPCOL )
+            NP0 = NUMROC( N+IROFFA, NB, 0, 0, NPROW )
+            MQ0 = NUMROC( N+ICOFFA, NB, 0, 0, NPCOL )
             IF( WANTZ ) THEN
-               NB_Z = DESCZ( NB_ )
-               MB_Z = DESCZ( MB_ )
                RSRC_Z = DESCZ( RSRC_ )
                IROFFZ = MOD( IZ-1, MB_A )
                IZROW = INDXG2P( 1, NB_A, MYROW, RSRC_Z, NPROW )
             END IF
 *
-            IF( ( .NOT.WANTZ ) .OR. (VALEIG.AND.(.NOT.LQUERY)) ) THEN
+            IF( ( .NOT.WANTZ ) .OR. ( VALEIG .AND. ( .NOT.LQUERY ) ) )
+     $           THEN
                LWMIN = N + MAX( NB*( NP0+1 ), 3 )
+               LWOPT = LWMIN
                LRWMIN = 5*NN + 4*N
+               IF( WANTZ ) THEN
+                  MQ0 = NUMROC( MAX( N, NB, 2 ), NB, 0, 0, NPCOL )
+                  LRWOPT = 4*N + MAX( 5*NN, NP0*MQ0 ) +
+     $                     ICEIL( N, NPROW*NPCOL )*NN
+               ELSE
+                  LRWOPT = LRWMIN
+               END IF
                NEIG = 0
             ELSE
                IF( ALLEIG .OR. VALEIG ) THEN
@@ -620,14 +628,24 @@
                ELSE IF( INDEIG ) THEN
                   NEIG = IU - IL + 1
                END IF
-               MQ0 = NUMROC( MAX( NEIG, NB, 2 ), NB, MYCOL, IACOL,
-     $                       NPCOL )
+               MQ0 = NUMROC( MAX( NEIG, NB, 2 ), NB, 0, 0, NPCOL )
                NQ0 = NUMROC( NN, NB, 0, 0, NPCOL )
                LWMIN = N + ( NP0+NQ0+NB )*NB
                LRWMIN = 4*N + MAX( 5*NN, NP0*MQ0 ) +
      $                  ICEIL( NEIG, NPROW*NPCOL )*NN
+               LRWOPT = LRWMIN
+               LWOPT = LWMIN
 *
             END IF
+*
+*           Conpute how much workspace is needed to use the
+*           new TRD code
+*
+            ANB = PJLAENV( ICTXT, 3, 'PZHETTRD', 'L', 0, 0, 0, 0 )
+            SQNPC = INT( SQRT( DBLE( NPROW*NPCOL ) ) )
+            NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+            NHETRD_LWOPT = 2*( ANB+1 )*( 4*NPS+2 ) + ( NPS+2 )*NPS
+            LWOPT = MAX( LWOPT, N+NHETRD_LWOPT )
 *
          END IF
          IF( INFO.EQ.0 ) THEN
@@ -640,11 +658,9 @@
                   RWORK( 2 ) = ZERO
                   RWORK( 3 ) = ZERO
                END IF
-               CALL DGEBS2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, RWORK,
-     $                         3 )
+               CALL DGEBS2D( ICTXT, 'ALL', ' ', 3, 1, RWORK, 3 )
             ELSE
-               CALL DGEBR2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, RWORK, 3,
-     $                       0, 0 )
+               CALL DGEBR2D( ICTXT, 'ALL', ' ', 3, 1, RWORK, 3, 0, 0 )
             END IF
             IF( .NOT.( WANTZ .OR. LSAME( JOBZ, 'N' ) ) ) THEN
                INFO = -1
@@ -662,9 +678,9 @@
                INFO = -12
             ELSE IF( LWORK.LT.LWMIN .AND. LWORK.NE.-1 ) THEN
                INFO = -23
-            ELSE IF( LRWORK.LT.LRWMIN .AND. LRWORK.NE.-1) THEN
+            ELSE IF( LRWORK.LT.LRWMIN .AND. LRWORK.NE.-1 ) THEN
                INFO = -25
-            ELSE IF( LIWORK.LT.LIWMIN .AND. LIWORK.NE.-1) THEN
+            ELSE IF( LIWORK.LT.LIWMIN .AND. LIWORK.NE.-1 ) THEN
                INFO = -27
             ELSE IF( VALEIG .AND. ( ABS( RWORK( 2 )-VL ).GT.FIVE*EPS*
      $               ABS( VL ) ) ) THEN
@@ -672,8 +688,8 @@
             ELSE IF( VALEIG .AND. ( ABS( RWORK( 3 )-VU ).GT.FIVE*EPS*
      $               ABS( VU ) ) ) THEN
                INFO = -10
-            ELSE IF( ABS( RWORK( 1 )-ABSTOL ).GT.FIVE*EPS*ABS( ABSTOL )
-     $                ) THEN
+            ELSE IF( ABS( RWORK( 1 )-ABSTOL ).GT.FIVE*EPS*
+     $               ABS( ABSTOL ) ) THEN
                INFO = -13
             ELSE IF( IROFFA.NE.0 ) THEN
                INFO = -6
@@ -697,7 +713,7 @@
                   INFO = -( 2100+RSRC_ )
                ELSE IF( DESCA( CSRC_ ).NE.DESCZ( CSRC_ ) ) THEN
                   INFO = -( 2100+CSRC_ )
-               ELSE IF( DESCA( CTXT_ ).NE.DESCZ( CTXT_ ) ) THEN
+               ELSE IF( ICTXT.NE.DESCZ( CTXT_ ) ) THEN
                   INFO = -( 2100+CTXT_ )
                END IF
             END IF
@@ -729,19 +745,19 @@
          END IF
          IDUM2( 4 ) = 4
          IF( WANTZ ) THEN
-            CALL PCHK2MAT( N, 3, N, 3, IA, JA, DESCA, 7, N, 3, N, 3,
-     $                     IZ, JZ, DESCZ, 12, 3, IDUM1, IDUM2, INFO )
+            CALL PCHK2MAT( N, 4, N, 4, IA, JA, DESCA, 8, N, 4, N, 4, IZ,
+     $                     JZ, DESCZ, 21, 4, IDUM1, IDUM2, INFO )
          ELSE
-            CALL PCHK1MAT( N, 3, N, 3, IA, JA, DESCA, 7, 3, IDUM1,
+            CALL PCHK1MAT( N, 4, N, 4, IA, JA, DESCA, 8, 4, IDUM1,
      $                     IDUM2, INFO )
          END IF
-         WORK( 1 ) = DCMPLX( LWMIN )
-         RWORK( 1 ) = DBLE( LRWMIN )
+         WORK( 1 ) = DCMPLX( LWOPT )
+         RWORK( 1 ) = DBLE( LRWOPT )
          IWORK( 1 ) = LIWMIN
       END IF
 *
       IF( INFO.NE.0 ) THEN
-         CALL PXERBLA( DESCA( CTXT_ ), 'PZHEEVX', -INFO )
+         CALL PXERBLA( ICTXT, 'PZHEEVX', -INFO )
          RETURN
       ELSE IF( LQUERY ) THEN
          RETURN
@@ -755,7 +771,7 @@
             ICLUSTR( 1 ) = 0
          END IF
          M = 0
-         WORK( 1 ) = DCMPLX( LWMIN )
+         WORK( 1 ) = DCMPLX( LWOPT )
          RWORK( 1 ) = DBLE( LRWMIN )
          IWORK( 1 ) = LIWMIN
          RETURN
@@ -773,8 +789,8 @@
          VUU = ZERO
       END IF
 *
-      ANRM = PZLANHE( 'M', UPLO, N, A, IA, JA, DESCA, 
-     $         RWORK( INDRWORK ) )
+      ANRM = PZLANHE( 'M', UPLO, N, A, IA, JA, DESCA,
+     $       RWORK( INDRWORK ) )
 *
       IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
          ISCALE = 1
@@ -799,14 +815,14 @@
          END IF
       END IF
 *
-*     Call PZHETRD to reduce Hermitian matrix to tridiagonal form.
+*     Call PZHENTRD to reduce Hermitian matrix to tridiagonal form.
 *
       LALLWORK = LLRWORK
 *
-      CALL PZHETRD( UPLO, N, A, IA, JA, DESCA, RWORK( INDD ),
-     $              RWORK( INDE ), WORK( INDTAU ), WORK( INDWORK ),
-     $              LLWORK, IINFO )
-
+      CALL PZHENTRD( UPLO, N, A, IA, JA, DESCA, RWORK( INDD ),
+     $               RWORK( INDE ), WORK( INDTAU ), WORK( INDWORK ),
+     $               LLWORK, RWORK( INDRWORK ), LLRWORK, IINFO )
+*
 *
 *     Copy the values of D, E to all processes
 *
@@ -815,50 +831,48 @@
 *     distributions so we have PxELGET as a backup.
 *
       OFFSET = 0
-      IF( IA.EQ.1 .AND. JA.EQ.1 .AND.
-     $    RSRC_A.EQ.0 .AND. CSRC_A.EQ.0 )THEN
-          CALL PDLARED1D( N, IA, JA, DESCA, RWORK( INDD ),
-     $                RWORK( INDD2 ), RWORK( INDRWORK ),
-     $                LLRWORK )
+      IF( IA.EQ.1 .AND. JA.EQ.1 .AND. RSRC_A.EQ.0 .AND. CSRC_A.EQ.0 )
+     $     THEN
+         CALL PDLARED1D( N, IA, JA, DESCA, RWORK( INDD ),
+     $                   RWORK( INDD2 ), RWORK( INDRWORK ), LLRWORK )
 *
-          CALL PDLARED1D( N, IA, JA, DESCA, RWORK( INDE ), 
-     $                RWORK( INDE2 ),  RWORK( INDRWORK ),
-     $                LLRWORK )
-          IF( .NOT.LOWER ) OFFSET = 1
+         CALL PDLARED1D( N, IA, JA, DESCA, RWORK( INDE ),
+     $                   RWORK( INDE2 ), RWORK( INDRWORK ), LLRWORK )
+         IF( .NOT.LOWER )
+     $      OFFSET = 1
       ELSE
-          DO 10 I=1,N
-             CALL PZELGET( 'A', ' ', WORK(INDD2+I-1), A,
-     $                 I+IA-1, I+JA-1, DESCA )
-             RWORK(INDD2+I-1) = DBLE( WORK(INDD2+I-1) )
-10        CONTINUE
-          IF( LSAME( UPLO, 'U') ) THEN
-              DO 20 I=1,N-1
-                 CALL PZELGET( 'A', ' ', WORK(INDE2+I-1), A, 
-     $                     I+IA-1, I+JA, DESCA )
-                 RWORK(INDE2+I-1) = DBLE( WORK(INDE2+I-1) )
-20            CONTINUE
-          ELSE
-              DO 30 I=1,N-1
-                 CALL PZELGET( 'A', ' ', WORK(INDE2+I-1), A,
-     $                     I+IA, I+JA-1, DESCA )
-                 RWORK(INDE2+I-1) = DBLE( WORK(INDE2+I-1) )
-30            CONTINUE
-          ENDIF
-      ENDIF
+         DO 10 I = 1, N
+            CALL PZELGET( 'A', ' ', WORK( INDD2+I-1 ), A, I+IA-1,
+     $                    I+JA-1, DESCA )
+            RWORK( INDD2+I-1 ) = DBLE( WORK( INDD2+I-1 ) )
+   10    CONTINUE
+         IF( LSAME( UPLO, 'U' ) ) THEN
+            DO 20 I = 1, N - 1
+               CALL PZELGET( 'A', ' ', WORK( INDE2+I-1 ), A, I+IA-1,
+     $                       I+JA, DESCA )
+               RWORK( INDE2+I-1 ) = DBLE( WORK( INDE2+I-1 ) )
+   20       CONTINUE
+         ELSE
+            DO 30 I = 1, N - 1
+               CALL PZELGET( 'A', ' ', WORK( INDE2+I-1 ), A, I+IA,
+     $                       I+JA-1, DESCA )
+               RWORK( INDE2+I-1 ) = DBLE( WORK( INDE2+I-1 ) )
+   30       CONTINUE
+         END IF
+      END IF
 *
 *     Call PDSTEBZ and, if eigenvectors are desired, PZSTEIN.
 *
       IF( WANTZ ) THEN
-         ORDER = 'b'
+         ORDER = 'B'
       ELSE
-         ORDER = 'e'
+         ORDER = 'E'
       END IF
 *
-      CALL PDSTEBZ( DESCA( CTXT_ ), RANGE, ORDER, N, VLL, VUU, IL, IU,
-     $              ABSTLL, RWORK( INDD2 ), RWORK( INDE2+OFFSET ), M,
-     $              NSPLIT, W, IWORK( INDIBL ), IWORK( INDISP ),
-     $              RWORK( INDRWORK ), LLRWORK, IWORK( 1 ), 
-     $              ISIZESTEBZ, IINFO )
+      CALL PDSTEBZ( ICTXT, RANGE, ORDER, N, VLL, VUU, IL, IU, ABSTLL,
+     $              RWORK( INDD2 ), RWORK( INDE2+OFFSET ), M, NSPLIT, W,
+     $              IWORK( INDIBL ), IWORK( INDISP ), RWORK( INDRWORK ),
+     $              LLRWORK, IWORK( 1 ), ISIZESTEBZ, IINFO )
 *
 *
 *     IF PDSTEBZ fails, the error propogates to INFO, but
@@ -888,18 +902,15 @@
 *           Since all processes must end up with the same value
 *           of NZ, we first compute the minimum of LALLWORK
 *
-            CALL IGAMN2D( DESCA( CTXT_ ), 'A', ' ', 1, 1, LALLWORK, 1,
-     $                    1, 1, -1, -1, -1 )
+            CALL IGAMN2D( ICTXT, 'A', ' ', 1, 1, LALLWORK, 1, 1, 1, -1,
+     $                    -1, -1 )
 *
             MAXEIGS = DESCZ( N_ )
 *
             DO 50 NZ = MIN( MAXEIGS, M ), 0, -1
                MQ0 = NUMROC( NZ, NB, 0, 0, NPCOL )
                SIZESTEIN = ICEIL( NZ, NPROCS )*N + MAX( 5*N, NP0*MQ0 )
-               SIZEORMTR = MAX( ( NB*( NB-1 ) ) / 2, ( MQ0+NP0 )*NB ) +
-     $                     NB*NB
-*
-               SIZEHEEVX = MAX( SIZESTEIN, SIZEORMTR )
+               SIZEHEEVX = SIZESTEIN
                IF( SIZEHEEVX.LE.LALLWORK )
      $            GO TO 60
    50       CONTINUE
@@ -926,14 +937,15 @@
 *
             IF( NSPLIT.GT.1 ) THEN
                CALL DLASRT( 'I', M, W, IINFO )
+               NZZ = 0
                IF( NZ.GT.0 ) THEN
 *
                   VUU = W( NZ ) - TEN*( EPS*ANRM+SAFMIN )
                   IF( VLL.GE.VUU ) THEN
                      NZZ = 0
                   ELSE
-                     CALL PDSTEBZ( DESCA( CTXT_ ), RANGE, ORDER, N, VLL,
-     $                             VUU, IL, IU, ABSTLL, RWORK( INDD2 ),
+                     CALL PDSTEBZ( ICTXT, RANGE, ORDER, N, VLL, VUU, IL,
+     $                             IU, ABSTLL, RWORK( INDD2 ),
      $                             RWORK( INDE2+OFFSET ), NZZ, NSPLIT,
      $                             W, IWORK( INDIBL ), IWORK( INDISP ),
      $                             RWORK( INDRWORK ), LLRWORK,
@@ -950,9 +962,9 @@
 *
             END IF
          END IF
-         CALL PZSTEIN( N, RWORK( INDD2 ), RWORK( INDE2+OFFSET ),
-     $                 NZ, W, IWORK( INDIBL ), IWORK( INDISP ), ORFAC,
-     $                 Z, IZ, JZ, DESCZ, RWORK( INDRWORK ), LALLWORK,
+         CALL PZSTEIN( N, RWORK( INDD2 ), RWORK( INDE2+OFFSET ), NZ, W,
+     $                 IWORK( INDIBL ), IWORK( INDISP ), ORFAC, Z, IZ,
+     $                 JZ, DESCZ, RWORK( INDRWORK ), LALLWORK,
      $                 IWORK( 1 ), ISIZESTEIN, IFAIL, ICLUSTR, GAP,
      $                 IINFO )
 *
@@ -978,8 +990,8 @@
          CALL DSCAL( M, ONE / SIGMA, W, 1 )
       END IF
 *
-      WORK( 1 ) = DCMPLX( LWMIN )
-      RWORK( 1 ) = DBLE( LRWMIN )
+      WORK( 1 ) = DCMPLX( LWOPT )
+      RWORK( 1 ) = DBLE( LRWOPT )
       IWORK( 1 ) = LIWMIN
 *
       RETURN
