@@ -1,15 +1,13 @@
-*
-*
       SUBROUTINE PZHEGVX( IBTYPE, JOBZ, RANGE, UPLO, N, A, IA, JA,
      $                    DESCA, B, IB, JB, DESCB, VL, VU, IL, IU,
      $                    ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ,
      $                    WORK, LWORK, RWORK, LRWORK, IWORK, LIWORK,
      $                    IFAIL, ICLUSTR, GAP, INFO )
 *
-*  -- ScaLAPACK routine (version 1.5) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     April 15, 1997
+*     October 15, 1999
 *
 *     .. Scalar Arguments ..
       CHARACTER          JOBZ, RANGE, UPLO
@@ -41,6 +39,7 @@
 *  Notes
 *  =====
 *
+*
 *  Each global data object is described by an associated description
 *  vector.  This vector stores the information required to establish
 *  the mapping between an object element and its corresponding process
@@ -92,6 +91,7 @@
 *          LOCr( M ) <= ceil( ceil(M/MB_A)/NPROW )*MB_A
 *          LOCc( N ) <= ceil( ceil(N/NB_A)/NPCOL )*NB_A
 *
+*
 *  Arguments
 *  =========
 *
@@ -136,12 +136,12 @@
 *          the diagonal, is destroyed.
 *
 *  IA      (global input) INTEGER
-*          A's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array A indicating the first
+*          row of sub( A ).
 *
 *  JA      (global input) INTEGER
-*          A's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array A indicating the
+*          first column of sub( A ).
 *
 *  DESCA   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix A.
@@ -163,12 +163,12 @@
 *          sub( B ) = L*L**H.
 *
 *  IB      (global input) INTEGER
-*          B's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array B indicating the first
+*          row of sub( B ).
 *
 *  JB      (global input) INTEGER
-*          B's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array B indicating the
+*          first column of sub( B ).
 *
 *  DESCB   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix B.
@@ -254,7 +254,8 @@
 *          ORFAC should be identical on all processes.
 *
 *  Z       (local output) COMPLEX*16 array,
-*          global dimension (N, N), local dimension ( LLD_Z, LOCc(JZ+N-1) )
+*          global dimension (N, N),
+*          local dimension ( LLD_Z, LOCc(JZ+N-1) )
 *          If JOBZ = 'V', then on normal exit the first M columns of Z
 *          contain the orthonormal eigenvectors of the matrix
 *          corresponding to the selected eigenvalues.  If an eigenvector
@@ -264,12 +265,12 @@
 *          If JOBZ = 'N', then Z is not referenced.
 *
 *  IZ      (global input) INTEGER
-*          Z's global row index, which points to the beginning of the
-*          submatrix which is to be operated on.
+*          The row index in the global array Z indicating the first
+*          row of sub( Z ).
 *
 *  JZ      (global input) INTEGER
-*          Z's global column index, which points to the beginning of
-*          the submatrix which is to be operated on.
+*          The column index in the global array Z indicating the
+*          first column of sub( Z ).
 *
 *  DESCZ   (global and local input) INTEGER array of dimension DLEN_.
 *          The array descriptor for the distributed matrix Z.
@@ -277,48 +278,53 @@
 *
 *  WORK    (local workspace/output) COMPLEX*16 array,
 *             dimension (LWORK)
-*          Version 1.0:  on output, WORK(1) returns the workspace
-*          needed to guarantee completion, but not orthogonality
-*          of the eigenvectors.
-*          If the input parameters are incorrect, WORK(1) may also be
-*          incorrect.
-*
-*          Later we will modify this so if enough workspace is given
-*          to complete the request, WORK(1) will return the amount
-*          of workspace needed to guarantee orthogonality.  This is
-*          described in great detail below:
-*          if INFO>=0, then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' WORK(1) = minimal workspace required to
-*               guarantee orthogonal eigenvectors on the given input
-*               matrix with the given ORFAC.  (In version 1.0 WORK(1) =
-*               minimal workspace required to compute eigenvalues.)
-*          if INFO <0 then
-*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
-*             if JOBZ='V' then
-*                if RANGE='A' or RANGE='I' then
-*                   WORK(1) = minimal workspace required to compute
-*                   all eigenvectors (no guarantee on orthogonality)
-*                if RANGE='V' then
-*                   WORK(1) = minimal workspace required to compute
-*                   N_Z = DESCZ(N_) eigenvectors (no guarantee on
-*                   orthogonality.)  (In version 1.0 WORK(1) =
-*                   minimal workspace required to compute eigenvalues.)
+*          WORK(1) returns the optimal workspace.
 *
 *  LWORK   (local input) INTEGER
 *          Size of WORK array.  If only eigenvalues are requested:
-*            LWORK >= N + ( NP0 + MQ0 + NB ) * NB
-*          If eigenvectors are requested:
 *            LWORK >= N + MAX( NB * ( NP0 + 1 ), 3 )
+*          If eigenvectors are requested:
+*            LWORK >= N + ( NP0 + MQ0 + NB ) * NB
+*          with NQ0 = NUMROC( NN, NB, 0, 0, NPCOL ).
+*
+*          For optimal performance, greater workspace is needed, i.e.
+*            LWORK >= MAX( LWORK, N + NHETRD_LWOPT,
+*              NHEGST_LWOPT )
+*          Where LWORK is as defined above, and
+*            NHETRD_LWORK = 2*( ANB+1 )*( 4*NPS+2 ) +
+*              ( NPS + 1 ) * NPS
+*            NHEGST_LWOPT =  2*NP0*NB + NQ0*NB + NB*NB
+*
+*            NB = DESCA( MB_ )
+*            NP0 = NUMROC( N, NB, 0, 0, NPROW )
+*            NQ0 = NUMROC( N, NB, 0, 0, NPCOL )
+*            ICTXT = DESCA( CTXT_ )
+*            ANB = PJLAENV( ICTXT, 3, 'PZHETTRD', 'L', 0, 0, 0, 0 )
+*            SQNPC = SQRT( DBLE( NPROW * NPCOL ) )
+*            NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+*
+*            NUMROC is a ScaLAPACK tool functions;
+*            PJLAENV is a ScaLAPACK envionmental inquiry function
+*            MYROW, MYCOL, NPROW and NPCOL can be determined by calling
+*            the subroutine BLACS_GRIDINFO.
 *
 *          If LWORK = -1, then LWORK is global input and a workspace
-*          query is assumed; the routine only calculates the minimum
-*          and optimal size for all work arrays. Each of these
-*          values is returned in the first entry of the corresponding
-*          work array, and no error message is issued by PXERBLA.
+*          query is assumed; the routine only calculates the optimal
+*          size for all work arrays. Each of these values is returned
+*          in the first entry of the correspondingwork array, and no
+*          error message is issued by PXERBLA.
 *
 *  RWORK   (local workspace/output) DOUBLE PRECISION array,
 *             dimension (LRWORK)
+*          On return, RWORK(1) contains the amount of workspace
+*             required for optimal efficiency
+*          if JOBZ='N' RWORK(1) = optimal amount of workspace
+*             required to compute eigenvalues efficiently
+*          if JOBZ='V' RWORK(1) = optimal amount of workspace
+*             required to compute eigenvalues and eigenvectors
+*             efficiently with no guarantee on orthogonality.
+*             If RANGE='V', it is assumed that all eigenvectors
+*             may be required when computing optimal workspace.
 *
 *  LRWORK  (local input) INTEGER
 *          Size of RWORK
@@ -391,14 +397,13 @@
 *  IWORK   (local workspace) INTEGER array
 *          On return, IWORK(1) contains the amount of integer workspace
 *          required.
-*          If the input parameters are incorrect, IWORK(1) may also be
-*          incorrect.
 *
 *  LIWORK  (local input) INTEGER
 *          size of IWORK
 *          LIWORK >= 6 * NNP
 *          Where:
 *            NNP = MAX( N, NPROW*NPCOL + 1, 4 )
+*
 *          If LIWORK = -1, then LIWORK is global input and a workspace
 *          query is assumed; the routine only calculates the minimum
 *          and optimal size for all work arrays. Each of these
@@ -501,10 +506,11 @@
 *     .. Local Scalars ..
       LOGICAL            ALLEIG, INDEIG, LQUERY, UPPER, VALEIG, WANTZ
       CHARACTER          TRANS
-      INTEGER            IACOL, IAROW, IBCOL, IBROW, ICOFFA, ICOFFB,
-     $                   ICTXT, IROFFA, IROFFB, LIWMIN, LRWMIN, LWMIN,
-     $                   MQ0, MYCOL, MYROW, NB, NEIG, NN, NP0, NPCOL,
-     $                   NPROW
+      INTEGER            ANB, IACOL, IAROW, IBCOL, IBROW, ICOFFA,
+     $                   ICOFFB, ICTXT, IROFFA, IROFFB, LIWMIN, LRWMIN,
+     $                   LRWOPT, LWMIN, LWOPT, MQ0, MYCOL, MYROW, NB,
+     $                   NEIG, NHEGST_LWOPT, NHETRD_LWOPT, NN, NP0,
+     $                   NPCOL, NPROW, NPS, NQ0, SQNPC
       DOUBLE PRECISION   EPS, SCALE
 *     ..
 *     .. Local Arrays ..
@@ -512,17 +518,18 @@
 *     ..
 *     .. External Functions ..
       LOGICAL            LSAME
-      INTEGER            ICEIL, INDXG2P, NUMROC
+      INTEGER            ICEIL, INDXG2P, NUMROC, PJLAENV
       DOUBLE PRECISION   PDLAMCH
-      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PDLAMCH
+      EXTERNAL           LSAME, ICEIL, INDXG2P, NUMROC, PJLAENV, PDLAMCH
 *     ..
 *     .. External Subroutines ..
       EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DGEBR2D, DGEBS2D,
      $                   DSCAL, PCHK1MAT, PCHK2MAT, PXERBLA, PZHEEVX,
-     $                   PZHEGST, PZPOTRF, PZTRMM, PZTRSM
+     $                   PZHENGST, PZPOTRF, PZTRMM, PZTRSM
 *     ..
 *     .. Intrinsic Functions ..
-      INTRINSIC          ABS, DBLE, DCMPLX, ICHAR, MAX, MIN, MOD
+      INTRINSIC          ABS, DBLE, DCMPLX, ICHAR, INT, MAX, MIN, MOD,
+     $                   SQRT
 *     ..
 *     .. Executable Statements ..
 *       This is just to keep ftnchek and toolpack/1 happy
@@ -570,8 +577,8 @@
                CALL DGEBS2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, RWORK,
      $                       3 )
             ELSE
-               CALL DGEBR2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, RWORK,
-     $                       3, 0, 0 )
+               CALL DGEBR2D( DESCA( CTXT_ ), 'ALL', ' ', 3, 1, RWORK, 3,
+     $                       0, 0 )
             END IF
             IAROW = INDXG2P( IA, DESCA( MB_ ), MYROW, DESCA( RSRC_ ),
      $              NPROW )
@@ -601,7 +608,14 @@
             IF( ( .NOT.WANTZ ) .OR. ( VALEIG .AND. ( .NOT.LQUERY ) ) )
      $           THEN
                LWMIN = N + MAX( NB*( NP0+1 ), 3 )
+               LWOPT = LWMIN
                LRWMIN = 5*NN + 4*N
+               IF( WANTZ ) THEN
+                  MQ0 = NUMROC( MAX( N, NB, 2 ), NB, 0, 0, NPCOL )
+                  LRWOPT = 4*N + MAX( 5*NN, NP0*MQ0 )
+               ELSE
+                  LRWOPT = LRWMIN
+               END IF
                NEIG = 0
             ELSE
                IF( ALLEIG .OR. VALEIG ) THEN
@@ -611,11 +625,26 @@
                END IF
                MQ0 = NUMROC( MAX( NEIG, NB, 2 ), NB, 0, 0, NPCOL )
                LWMIN = N + ( NP0+MQ0+NB )*NB
+               LWOPT = LWMIN
                LRWMIN = 4*N + MAX( 5*NN, NP0*MQ0 ) +
      $                  ICEIL( NEIG, NPROW*NPCOL )*NN
+               LRWOPT = LRWMIN
 *
             END IF
 *
+*           Conpute how much workspace is needed to use the
+*           new TRD and GST algorithms
+*
+            ANB = PJLAENV( ICTXT, 3, 'PZHETTRD', 'L', 0, 0, 0, 0 )
+            SQNPC = INT( SQRT( DBLE( NPROW*NPCOL ) ) )
+            NPS = MAX( NUMROC( N, 1, 0, 0, SQNPC ), 2*ANB )
+            NHETRD_LWOPT = 2*( ANB+1 )*( 4*NPS+2 ) + ( NPS+4 )*NPS
+            NB = DESCA( MB_ )
+            NP0 = NUMROC( N, NB, 0, 0, NPROW )
+            NQ0 = NUMROC( N, NB, 0, 0, NPCOL )
+            NHEGST_LWOPT = 2*NP0*NB + NQ0*NB + NB*NB
+            LWOPT = MAX( LWOPT, N+NHETRD_LWOPT, NHEGST_LWOPT )
+*
 *       Version 1.0 Limitations
 *
             IF( IBTYPE.LT.1 .OR. IBTYPE.GT.3 ) THEN
@@ -669,25 +698,25 @@
             ELSE IF( VALEIG .AND. N.GT.0 .AND. VU.LE.VL ) THEN
                INFO = -15
             ELSE IF( INDEIG .AND. ( IL.LT.1 .OR. IL.GT.MAX( 1, N ) ) )
-     $               THEN
+     $                THEN
                INFO = -16
             ELSE IF( INDEIG .AND. ( IU.LT.MIN( N, IL ) .OR. IU.GT.N ) )
-     $               THEN
+     $                THEN
                INFO = -17
             ELSE IF( VALEIG .AND. ( ABS( RWORK( 2 )-VL ).GT.FIVE*EPS*
-     $              ABS( VL ) ) ) THEN
+     $               ABS( VL ) ) ) THEN
                INFO = -14
             ELSE IF( VALEIG .AND. ( ABS( RWORK( 3 )-VU ).GT.FIVE*EPS*
-     $              ABS( VU ) ) ) THEN
+     $               ABS( VU ) ) ) THEN
                INFO = -15
             ELSE IF( ABS( RWORK( 1 )-ABSTOL ).GT.FIVE*EPS*
-     $              ABS( ABSTOL ) ) THEN
+     $               ABS( ABSTOL ) ) THEN
                INFO = -18
-            ELSE IF( LWORK.LT.LWMIN .AND. LWORK.NE.-1 ) THEN
+            ELSE IF( LWORK.LT.LWMIN .AND. .NOT.LQUERY ) THEN
                INFO = -28
-            ELSE IF( LRWORK.LT.LRWMIN .AND. LRWORK.NE.-1 ) THEN
+            ELSE IF( LRWORK.LT.LRWMIN .AND. .NOT.LQUERY ) THEN
                INFO = -30
-            ELSE IF( LIWORK.LT.LIWMIN .AND. LIWORK.NE.-1 ) THEN
+            ELSE IF( LIWORK.LT.LIWMIN .AND. .NOT.LQUERY ) THEN
                INFO = -32
             END IF
          END IF
@@ -725,9 +754,9 @@
      $                  INFO )
       END IF
 *
-      WORK( 1 ) = DCMPLX( DBLE( LWMIN ) )
-      RWORK( 1 ) = DBLE( LRWMIN )
       IWORK( 1 ) = LIWMIN
+      WORK( 1 ) = DCMPLX( DBLE( LWOPT ) )
+      RWORK( 1 ) = DBLE( LRWOPT )
 *
       IF( INFO.NE.0 ) THEN
          CALL PXERBLA( ICTXT, 'PZHEGVX ', -INFO )
@@ -740,6 +769,9 @@
 *
       CALL PZPOTRF( UPLO, N, B, IB, JB, DESCB, INFO )
       IF( INFO.NE.0 ) THEN
+         IWORK( 1 ) = LIWMIN
+         WORK( 1 ) = DCMPLX( DBLE( LWOPT ) )
+         RWORK( 1 ) = DBLE( LRWOPT )
          IFAIL( 1 ) = INFO
          INFO = IERRNPD
          RETURN
@@ -747,12 +779,12 @@
 *
 *     Transform problem to standard eigenvalue problem and solve.
 *
-      CALL PZHEGST( IBTYPE, UPLO, N, A, IA, JA, DESCA, B, IB, JB,
-     $              DESCB, SCALE, INFO )
+      CALL PZHENGST( IBTYPE, UPLO, N, A, IA, JA, DESCA, B, IB, JB,
+     $               DESCB, SCALE, WORK, LWORK, INFO )
       CALL PZHEEVX( JOBZ, RANGE, UPLO, N, A, IA, JA, DESCA, VL, VU, IL,
-     $              IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ,
-     $              WORK, LWORK, RWORK, LRWORK, IWORK, LIWORK, IFAIL,
-     $              ICLUSTR, GAP, INFO )
+     $              IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ, JZ, DESCZ, WORK,
+     $              LWORK, RWORK, LRWORK, IWORK, LIWORK, IFAIL, ICLUSTR,
+     $              GAP, INFO )
 *
       IF( WANTZ ) THEN
 *
@@ -794,6 +826,9 @@
          CALL DSCAL( N, SCALE, W, 1 )
       END IF
 *
+      IWORK( 1 ) = LIWMIN
+      WORK( 1 ) = DCMPLX( DBLE( LWOPT ) )
+      RWORK( 1 ) = DBLE( LRWOPT )
       RETURN
 *
 *     End of PZHEGVX
