@@ -1,4 +1,4 @@
-*
+
 *
       SUBROUTINE PCSEPTST( DESCA, UPLO, N, MATTYPE, SUBTESTS, THRESH,
      $                     ORDER, ABSTOL, ISEED, A, COPYA, Z, LDA, WIN,
@@ -6,7 +6,7 @@
      $                     WORK, LWORK, RWORK, LRWORK, IWORK, LIWORK,
      $                     NOUT, INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
 *     November 15, 1997
@@ -225,13 +225,15 @@
       CHARACTER          JOBZ, RANGE
       CHARACTER*14       PASSED
       INTEGER            CONTEXT, I, IAM, IINFO, IL, IMODE, IN, INDD,
-     $                   INDRWORK, INDWORK, ISIZEHEEVX, ISIZESUBTST,
-     $                   ISIZETST, ITYPE, IU, J, LHEEVXSIZE, LLRWORK,
-     $                   LLWORK, MAXSIZE, MYCOL, MYROW, NB, NGEN, NLOC,
-     $                   NNODES, NP, NPCOL, NPROW, NQ, RES, RSIZECHK,
-     $                   RSIZEHEEVX, RSIZEQTQ, RSIZESUBTST, RSIZETST,
-     $                   SIZEHEEVX, SIZEMQRLEFT, SIZEMQRRIGHT, SIZEQRF,
-     $                   SIZESUBTST, SIZETMS, SIZETST, VALSIZE, VECSIZE
+     $                   INDRWORK, INDWORK, ISIZEHEEVD, ISIZEHEEVX,
+     $                   ISIZESUBTST, ISIZETST, ITYPE, IU, J,
+     $                   LHEEVDSIZE, LHEEVXSIZE, LLRWORK, LLWORK,
+     $                   MAXSIZE, MYCOL, MYROW, NB, NGEN, NLOC, NNODES,
+     $                   NP, NP0, NPCOL, NPROW, NQ, NQ0, RES, RSIZECHK,
+     $                   RSIZEHEEVD, RSIZEHEEVX, RSIZEQTQ, RSIZESUBTST,
+     $                   RSIZETST, SIZEHEEVD, SIZEHEEVX, SIZEMQRLEFT,
+     $                   SIZEMQRRIGHT, SIZEQRF, SIZESUBTST, SIZETMS,
+     $                   SIZETST, VALSIZE, VECSIZE
       REAL               ANINV, ANORM, COND, MAXQTQNRM, MAXTSTNRM, OVFL,
      $                   QTQNRM, RTOVFL, RTUNFL, TEMP1, TSTNRM, ULP,
      $                   ULPINV, UNFL, VL, VU
@@ -251,8 +253,8 @@
       EXTERNAL           BLACS_GRIDINFO, BLACS_PINFO, CLATMS, IGAMX2D,
      $                   IGEBR2D, IGEBS2D, PCCHEKPAD, PCELSET,
      $                   PCFILLPAD, PCLASET, PCLASIZEHEEVX, PCLASIZESEP,
-     $                   PCLATMS, PCMATGEN, PCSEPSUBTST, SLABAD, SLASRT,
-     $                   SLCOMBINE
+     $                   PCLATMS, PCMATGEN, PCSDPSUBTST, PCSEPSUBTST,
+     $                   SLABAD, SLASRT, SLCOMBINE
 *     ..
 *     .. Intrinsic Functions ..
       INTRINSIC          ABS, INT, MAX, MIN, REAL, SQRT
@@ -271,7 +273,7 @@
      $    RSRC_.LT.0 )RETURN
 *
       INFO = 0
-      PASSED = 'PASSED      '
+      PASSED = 'PASSED  EEVX'
       CONTEXT = DESCA( CTXT_ )
       NB = DESCA( NB_ )
 *
@@ -285,8 +287,9 @@
       CALL PCLASIZESEP( DESCA, IPREPAD, IPOSTPAD, SIZEMQRLEFT,
      $                  SIZEMQRRIGHT, SIZEQRF, SIZETMS, RSIZEQTQ,
      $                  RSIZECHK, SIZEHEEVX, RSIZEHEEVX, ISIZEHEEVX,
-     $                  SIZESUBTST, RSIZESUBTST, ISIZESUBTST, SIZETST,
-     $                  RSIZETST, ISIZETST )
+     $                  SIZEHEEVD, RSIZEHEEVD, ISIZEHEEVD, SIZESUBTST,
+     $                  RSIZESUBTST, ISIZESUBTST, SIZETST, RSIZETST,
+     $                  ISIZETST )
 *
       IF( LRWORK.LT.RSIZETST ) THEN
          INFO = 3
@@ -553,7 +556,7 @@
      $                       ISEED, RWORK( INDD ), MAXSIZE, VECSIZE,
      $                       VALSIZE )
 *
-         LHEEVXSIZE = MIN( MAXSIZE, LRWORK )
+         LHEEVXSIZE = MIN( MAXSIZE, LLRWORK )
 *
          CALL PCSEPSUBTST( WKNOWN, 'v', 'a', UPLO, N, VL, VU, IL, IU,
      $                     THRESH, ABSTOL, A, COPYA, Z, 1, 1, DESCA,
@@ -1048,6 +1051,92 @@
          END IF
       END IF
 *
+*     Now that PCHEEVX been tested, we check PCHEEVD
+*
+      PASSED = 'PASSED  EEVD'
+*
+*     PCHEEVD test1:
+*
+      IF( INFO.EQ.0 ) THEN
+*
+         NP0 = NUMROC( N, NB, 0, 0, NPROW )
+         NQ0 = NUMROC( MAX( N, 1 ), NB, 0, 0, NPCOL )
+         LHEEVDSIZE = MAX( 1, 7*N+3*NP0*NQ0 )
+         ISIZEHEEVD = MAX( 1, 2+7*N+8*NPCOL )
+*
+         CALL PCSDPSUBTST( WKNOWN, UPLO, N, THRESH, ABSTOL, A, COPYA, Z,
+     $                     1, 1, DESCA, WIN, WNEW, IPREPAD, IPOSTPAD,
+     $                     WORK( INDWORK ), LLWORK, RWORK, LRWORK,
+     $                     LHEEVDSIZE, IWORK, ISIZEHEEVD, RES, TSTNRM,
+     $                     QTQNRM, NOUT )
+*
+         MAXTSTNRM = TSTNRM
+         MAXQTQNRM = QTQNRM
+         
+         IF( RES.NE.0 ) THEN
+            PASSED = 'FAILED EEVD'
+            INFO = 1
+         END IF
+      END IF
+*
+*
+*
+      CALL IGAMX2D( CONTEXT, 'All', ' ', 1, 1, INFO, 1, -1, -1, -1, -1,
+     $              -1 )
+*
+      IF( INFO.EQ.1 ) THEN
+         IF( IAM.EQ.0 ) THEN
+            WRITE( NOUT, FMT = 9994 )'C  '
+            WRITE( NOUT, FMT = 9993 )ISEEDIN( 1 )
+            WRITE( NOUT, FMT = 9992 )ISEEDIN( 2 )
+            WRITE( NOUT, FMT = 9991 )ISEEDIN( 3 )
+            WRITE( NOUT, FMT = 9990 )ISEEDIN( 4 )
+            IF( LSAME( UPLO, 'L' ) ) THEN
+               WRITE( NOUT, FMT = 9994 )'      UPLO= ''L'' '
+            ELSE
+               WRITE( NOUT, FMT = 9994 )'      UPLO= ''U'' '
+            END IF
+            IF( LSAME( SUBTESTS, 'Y' ) ) THEN
+               WRITE( NOUT, FMT = 9994 )'      SUBTESTS= ''Y'' '
+            ELSE
+               WRITE( NOUT, FMT = 9994 )'      SUBTESTS= ''N'' '
+            END IF
+            WRITE( NOUT, FMT = 9989 )N
+            WRITE( NOUT, FMT = 9988 )NPROW
+            WRITE( NOUT, FMT = 9987 )NPCOL
+            WRITE( NOUT, FMT = 9986 )NB
+            WRITE( NOUT, FMT = 9985 )MATTYPE
+            WRITE( NOUT, FMT = 9982 )ABSTOL
+            WRITE( NOUT, FMT = 9981 )THRESH
+            WRITE( NOUT, FMT = 9994 )'C  '
+         END IF
+      END IF
+*
+      CALL SLCOMBINE( CONTEXT, 'All', '>', 'W', 6, 1, WTIME )
+      CALL SLCOMBINE( CONTEXT, 'All', '>', 'C', 6, 1, CTIME )
+      IF( IAM.EQ.0 ) THEN
+         IF( INFO.EQ.0 .OR. INFO.EQ.1 ) THEN
+            IF( WTIME( 1 ).GE.0.0 ) THEN
+               WRITE( NOUT, FMT = 9999 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, WTIME( 1 ), CTIME( 1 ), MAXTSTNRM,
+     $            MAXQTQNRM, PASSED
+            ELSE
+               WRITE( NOUT, FMT = 9998 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, CTIME( 1 ), MAXTSTNRM, MAXQTQNRM, PASSED
+            END IF
+         ELSE IF( INFO.EQ.2 ) THEN
+            IF( WTIME( 1 ).GE.0.0 ) THEN
+               WRITE( NOUT, FMT = 9997 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, WTIME( 1 ), CTIME( 1 )
+            ELSE
+               WRITE( NOUT, FMT = 9996 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, CTIME( 1 )
+            END IF
+         ELSE IF( INFO.EQ.3 ) THEN
+            WRITE( NOUT, FMT = 9995 )N, NB, NPROW, NPCOL, MATTYPE,
+     $         SUBTESTS
+         END IF
+      END IF
   120 CONTINUE
 *
       RETURN
