@@ -1,15 +1,13 @@
-*
-*
       SUBROUTINE PDSEPTST( DESCA, UPLO, N, MATTYPE, SUBTESTS, THRESH,
      $                     ORDER, ABSTOL, ISEED, A, COPYA, Z, LDA, WIN,
      $                     WNEW, IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
      $                     WORK, LWORK, IWORK, LIWORK, HETERO, NOUT,
      $                     INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
-*     November 15, 1997
+*     August 14, 2001
 *
 *     .. Scalar Arguments ..
       CHARACTER          HETERO, SUBTESTS, UPLO
@@ -220,7 +218,7 @@
      $                   NNODES, NP, NPCOL, NPROW, NQ, RES, SIZECHK, 
      $                   SIZEMQRLEFT, SIZEMQRRIGHT, SIZEQRF, SIZEQTQ, 
      $                   SIZESUBTST, SIZESYEV, SIZESYEVX, SIZETMS,
-     $                   SIZETST, VALSIZE, VECSIZE
+     $                   SIZETST, VALSIZE, VECSIZE, ISIZESYEVD,SIZESYEVD
       DOUBLE PRECISION   ANINV, ANORM, COND, MAXQTQNRM, MAXTSTNRM, OVFL, 
      $                   QTQNRM, RTOVFL, RTUNFL, TEMP1, TSTNRM, ULP, 
      $                   ULPINV, UNFL, VL, VU
@@ -289,8 +287,9 @@
 *
       CALL PDLASIZESQP( DESCA, IPREPAD, IPOSTPAD, SIZEMQRLEFT,
      $                  SIZEMQRRIGHT, SIZEQRF, SIZETMS, SIZEQTQ,
-     $                  SIZECHK, SIZESYEVX, ISIZESYEVX, SIZESYEV, 
-     $                  SIZESUBTST, ISIZESUBTST, SIZETST, ISIZETST )
+     $                  SIZECHK, SIZESYEVX, ISIZESYEVX, SIZESYEV,
+     $                  SIZESYEVD, ISIZESYEVD, SIZESUBTST, 
+     $                  ISIZESUBTST, SIZETST, ISIZETST )
 *
       IF( LWORK.LT.SIZETST ) THEN
          INFO = 3
@@ -553,7 +552,7 @@
      $                       ISEED, WORK( INDD ), MAXSIZE, VECSIZE,
      $                       VALSIZE )
 *
-         LSYEVXSIZE = MIN( MAXSIZE, LWORK )
+         LSYEVXSIZE = MIN( MAXSIZE, LLWORK )
 *
          CALL PDSEPSUBTST( WKNOWN, 'v', 'a', UPLO, N, VL, VU, IL, IU,
      $                     THRESH, ABSTOL, A, COPYA, Z, 1, 1, DESCA,
@@ -1061,13 +1060,18 @@
 *        PDSYEV test1:
 *        JOBZ = 'N', eigenvalues only
 *
-         IF( INFO.EQ.0 ) THEN
+         IF( INFO.NE.0 ) THEN
+*
+*           If the EVX tests fail, we do not perform the EV tests
+*
+            PASSED = 'SKIPPED   EV'
+         ELSE
             JOBZ = 'N'
 *
             CALL PDSYEV( JOBZ, UPLO, N, A, 1, 1, DESCA,
      $                  WORK( INDWORK ), Z, 1, 1, DESCA,
      $                  WORK( INDWORK ), -1, INFO )
-            MINSIZE = WORK( INDWORK )
+            MINSIZE = INT( WORK( INDWORK ) )
 *
             CALL PDSQPSUBTST( WKNOWN, JOBZ, UPLO, N, THRESH, ABSTOL, A,
      $                        COPYA, Z, 1, 1, DESCA, WIN, WNEW, IPREPAD,
@@ -1091,7 +1095,7 @@
             CALL PDSYEV( JOBZ, UPLO, N, A, 1, 1, DESCA,
      $                  WORK( INDWORK ), Z, 1, 1, DESCA,
      $                  WORK( INDWORK ), -1, INFO )
-            MINSIZE = WORK( INDWORK )
+            MINSIZE = INT( WORK( INDWORK ) )
 *
             CALL PDSQPSUBTST( WKNOWN, JOBZ, UPLO, N, THRESH, ABSTOL, A,
      $                        COPYA, Z, 1, 1, DESCA, WIN, WNEW, IPREPAD,
@@ -1139,6 +1143,90 @@
                ELSE
                   WRITE( NOUT, FMT = 9998 )N, NB, NPROW, NPCOL, MATTYPE,
      $                   SUBTESTS, CTIME( 1 ), MAXTSTNRM, MAXQTQNRM,
+     $                   PASSED
+               END IF
+            ELSE IF( INFO.EQ.2 ) THEN
+               IF( WTIME( 1 ).GE.0.0 ) THEN
+                  WRITE( NOUT, FMT = 9997 )N, NB, NPROW, NPCOL, MATTYPE,
+     $                   SUBTESTS, WTIME( 1 ), CTIME( 1 )
+               ELSE
+                  WRITE( NOUT, FMT = 9996 )N, NB, NPROW, NPCOL, MATTYPE,
+     $                   SUBTESTS, CTIME( 1 )
+               END IF
+            ELSE IF( INFO.EQ.3 ) THEN
+               WRITE( NOUT, FMT = 9995 )N, NB, NPROW, NPCOL, MATTYPE,
+     $                SUBTESTS
+            END IF
+         END IF
+      ENDIF
+*
+*     Now that PDSYEV been tested, we check PDSYEVD if we are a
+*     homogeneous machine.
+*
+      IF( LSAME( HETERO, 'N' ) .AND. LSAME( SUBTESTS, 'N' ) ) THEN
+         PASSED = 'PASSED   EVD'
+*
+*        PDSYEVD test1:
+*
+         IF( INFO.NE.0 ) THEN
+*
+*           If the EV tests fail, we do not perform the EVD tests
+*
+            PASSED = 'SKIPPED   EVD'
+         ELSE
+*
+            NP = NUMROC( N, DESCA( MB_ ), 0, 0, NPROW )
+            NQ = NUMROC( N, DESCA( NB_ ), 0, 0, NPCOL )
+            MINSIZE  = MAX( 1+6*N+2*NP*NQ, 
+     $           3*N + MAX( NB*( NP+1 ), 3*NB ) ) + 2*N
+*
+            CALL PDSDPSUBTST( WKNOWN, UPLO, N, THRESH, ABSTOL, A,
+     $                        COPYA, Z, 1, 1, DESCA, WIN, WNEW, IPREPAD,
+     $                        IPOSTPAD, WORK( INDWORK ), LLWORK,
+     $                        MINSIZE, IWORK, ISIZESYEVD,
+     $                        RES, TSTNRM, QTQNRM, NOUT )
+*
+            IF( RES.NE.0 ) THEN
+               MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
+               MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
+               PASSED = 'FAIL EVD test1'
+               INFO = 1
+            END IF
+         END IF
+         IF( INFO.EQ.1 ) THEN
+            IF( IAM.EQ.0 ) THEN
+               WRITE( NOUT, FMT = 9994 )'C  '
+               WRITE( NOUT, FMT = 9993 )ISEEDIN( 1 )
+               WRITE( NOUT, FMT = 9992 )ISEEDIN( 2 )
+               WRITE( NOUT, FMT = 9991 )ISEEDIN( 3 )
+               WRITE( NOUT, FMT = 9990 )ISEEDIN( 4 )
+               IF( LSAME( UPLO, 'L' ) ) THEN
+                  WRITE( NOUT, FMT = 9994 )'      UPLO= ''L'' '
+               ELSE
+                  WRITE( NOUT, FMT = 9994 )'      UPLO= ''U'' '
+               END IF
+               WRITE( NOUT, FMT = 9989 )N
+               WRITE( NOUT, FMT = 9988 )NPROW
+               WRITE( NOUT, FMT = 9987 )NPCOL
+               WRITE( NOUT, FMT = 9986 )NB
+               WRITE( NOUT, FMT = 9985 )MATTYPE
+               WRITE( NOUT, FMT = 9982 )ABSTOL
+               WRITE( NOUT, FMT = 9981 )THRESH
+               WRITE( NOUT, FMT = 9994 )'C  '
+            END IF
+         END IF
+*
+         CALL SLCOMBINE( CONTEXT, 'All', '>', 'W', 6, 1, WTIME )
+         CALL SLCOMBINE( CONTEXT, 'All', '>', 'C', 6, 1, CTIME )
+         IF( IAM.EQ.0 ) THEN
+            IF( INFO.EQ.0 .OR. INFO.EQ.1 ) THEN
+               IF( WTIME( 1 ).GE.0.0 ) THEN
+                  WRITE( NOUT, FMT = 9999 )N, NB, NPROW, NPCOL, MATTYPE,
+     $                   SUBTESTS, WTIME( 1 ), CTIME( 1 ), TSTNRM,
+     $                   QTQNRM, PASSED
+               ELSE
+                  WRITE( NOUT, FMT = 9998 )N, NB, NPROW, NPCOL, MATTYPE,
+     $                   SUBTESTS, CTIME( 1 ), MAXTSTNRM, MAXQTQNRM,
      $                   PASSED
                END IF
             ELSE IF( INFO.EQ.2 ) THEN
