@@ -6,7 +6,7 @@
      $                     WORK, LWORK, RWORK, LRWORK, IWORK, LIWORK,
      $                     NOUT, INFO )
 *
-*  -- ScaLAPACK routine (version 1.6) --
+*  -- ScaLAPACK routine (version 1.7) --
 *     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
 *     and University of California, Berkeley.
 *     November 15, 1997
@@ -231,7 +231,9 @@
      $                   NNODES, NP, NPCOL, NPROW, NQ, RES, RSIZECHK,
      $                   RSIZEHEEVX, RSIZEQTQ, RSIZESUBTST, RSIZETST,
      $                   SIZEHEEVX, SIZEMQRLEFT, SIZEMQRRIGHT, SIZEQRF,
-     $                   SIZESUBTST, SIZETMS, SIZETST, VALSIZE, VECSIZE
+     $                   SIZESUBTST, SIZETMS, SIZETST, VALSIZE, VECSIZE,
+     $                   SIZEHEEVD, RSIZEHEEVD, ISIZEHEEVD, NQ0, NP0,
+     $                   LHEEVDSIZE
       DOUBLE PRECISION   ANINV, ANORM, COND, MAXQTQNRM, MAXTSTNRM, OVFL,
      $                   QTQNRM, RTOVFL, RTUNFL, TEMP1, TSTNRM, ULP,
      $                   ULPINV, UNFL, VL, VU
@@ -271,7 +273,7 @@
      $    RSRC_.LT.0 )RETURN
 *
       INFO = 0
-      PASSED = 'PASSED      '
+      PASSED = 'PASSED   EVX'
       CONTEXT = DESCA( CTXT_ )
       NB = DESCA( NB_ )
 *
@@ -285,6 +287,7 @@
       CALL PZLASIZESEP( DESCA, IPREPAD, IPOSTPAD, SIZEMQRLEFT,
      $                  SIZEMQRRIGHT, SIZEQRF, SIZETMS, RSIZEQTQ,
      $                  RSIZECHK, SIZEHEEVX, RSIZEHEEVX, ISIZEHEEVX,
+     $                  SIZEHEEVD, RSIZEHEEVD, ISIZEHEEVD,
      $                  SIZESUBTST, RSIZESUBTST, ISIZESUBTST, SIZETST,
      $                  RSIZETST, ISIZETST )
 *
@@ -553,7 +556,7 @@
      $                       ISEED, RWORK( INDD ), MAXSIZE, VECSIZE,
      $                       VALSIZE )
 *
-         LHEEVXSIZE = MIN( MAXSIZE, LRWORK )
+         LHEEVXSIZE = MIN( MAXSIZE, LLRWORK )
 *
          CALL PZSEPSUBTST( WKNOWN, 'v', 'a', UPLO, N, VL, VU, IL, IU,
      $                     THRESH, ABSTOL, A, COPYA, Z, 1, 1, DESCA,
@@ -568,6 +571,7 @@
          MAXTSTNRM = TSTNRM
          MAXQTQNRM = QTQNRM
 *
+         res =0
          IF( THRESH.LE.ZERO ) THEN
             PASSED = 'SKIPPED       '
             INFO = 2
@@ -1048,6 +1052,92 @@
          END IF
       END IF
 *
+*     Now that PZHEEVX been tested, we check PZHEEVD
+*
+      PASSED = 'PASSED  EEVD'
+*
+*     PZHEEVD test1:
+*
+      IF( INFO.EQ.0 ) THEN
+*
+         NP0 = NUMROC( N, NB, 0, 0, NPROW )
+         NQ0 = NUMROC( MAX( N, 1 ), NB, 0, 0, NPCOL )
+         LHEEVDSIZE = MAX( 1, 7*N+3*NP0*NQ0 )
+         ISIZEHEEVD = MAX( 1, 2+7*N+8*NPCOL )
+*
+         CALL PZSDPSUBTST( WKNOWN, UPLO, N, THRESH, ABSTOL, A, COPYA, Z,
+     $                     1, 1, DESCA, WIN, WNEW, IPREPAD, IPOSTPAD,
+     $                     WORK( INDWORK ), LLWORK, RWORK, LRWORK,
+     $                     LHEEVDSIZE, IWORK, ISIZEHEEVD, RES, TSTNRM,
+     $                     QTQNRM, NOUT )
+*
+         MAXTSTNRM = TSTNRM
+         MAXQTQNRM = QTQNRM
+*
+         IF( RES.NE.0 ) THEN
+            PASSED = 'FAILED EEVD'
+            INFO = 1
+         END IF
+      END IF
+*
+*
+*
+      CALL IGAMX2D( CONTEXT, 'All', ' ', 1, 1, INFO, 1, -1, -1, -1, -1,
+     $              -1 )
+*
+      IF( INFO.EQ.1 ) THEN
+         IF( IAM.EQ.0 ) THEN
+            WRITE( NOUT, FMT = 9994 )'C  '
+            WRITE( NOUT, FMT = 9993 )ISEEDIN( 1 )
+            WRITE( NOUT, FMT = 9992 )ISEEDIN( 2 )
+            WRITE( NOUT, FMT = 9991 )ISEEDIN( 3 )
+            WRITE( NOUT, FMT = 9990 )ISEEDIN( 4 )
+            IF( LSAME( UPLO, 'L' ) ) THEN
+               WRITE( NOUT, FMT = 9994 )'      UPLO= ''L'' '
+            ELSE
+               WRITE( NOUT, FMT = 9994 )'      UPLO= ''U'' '
+            END IF
+            IF( LSAME( SUBTESTS, 'Y' ) ) THEN
+               WRITE( NOUT, FMT = 9994 )'      SUBTESTS= ''Y'' '
+            ELSE
+               WRITE( NOUT, FMT = 9994 )'      SUBTESTS= ''N'' '
+            END IF
+            WRITE( NOUT, FMT = 9989 )N
+            WRITE( NOUT, FMT = 9988 )NPROW
+            WRITE( NOUT, FMT = 9987 )NPCOL
+            WRITE( NOUT, FMT = 9986 )NB
+            WRITE( NOUT, FMT = 9985 )MATTYPE
+            WRITE( NOUT, FMT = 9982 )ABSTOL
+            WRITE( NOUT, FMT = 9981 )THRESH
+            WRITE( NOUT, FMT = 9994 )'C  '
+         END IF
+      END IF
+*
+      CALL SLCOMBINE( CONTEXT, 'All', '>', 'W', 6, 1, WTIME )
+      CALL SLCOMBINE( CONTEXT, 'All', '>', 'C', 6, 1, CTIME )
+      IF( IAM.EQ.0 ) THEN
+         IF( INFO.EQ.0 .OR. INFO.EQ.1 ) THEN
+            IF( WTIME( 1 ).GE.0.0 ) THEN
+               WRITE( NOUT, FMT = 9999 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, WTIME( 1 ), CTIME( 1 ), MAXTSTNRM,
+     $            MAXQTQNRM, PASSED
+            ELSE
+               WRITE( NOUT, FMT = 9998 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, CTIME( 1 ), MAXTSTNRM, MAXQTQNRM, PASSED
+            END IF
+         ELSE IF( INFO.EQ.2 ) THEN
+            IF( WTIME( 1 ).GE.0.0 ) THEN
+               WRITE( NOUT, FMT = 9997 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, WTIME( 1 ), CTIME( 1 )
+            ELSE
+               WRITE( NOUT, FMT = 9996 )N, NB, NPROW, NPCOL, MATTYPE,
+     $            SUBTESTS, CTIME( 1 )
+            END IF
+         ELSE IF( INFO.EQ.3 ) THEN
+            WRITE( NOUT, FMT = 9995 )N, NB, NPROW, NPCOL, MATTYPE,
+     $         SUBTESTS
+         END IF
+      END IF
   120 CONTINUE
 *
       RETURN
